# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

# importing Magics module
from Magics.macro import *

# Definitions of the projection using a dictionary to 
# organise the informations
projection = { "name": "projection1",
				"project" : "global",
                "text" : "Global Cylindrical",
                "map" : mmap(
                  subpage_map_projection      = "cylindrical",
                  subpage_upper_right_longitude= 180.,
                  subpage_upper_right_latitude= 90.,
                  subpage_lower_left_longitude =  -180.,
                  subpage_lower_left_latitude = -90.)
              }


# Setting of the output file name
output = output(output_name=projection["name"],
				output_formats = ['png'],
				output_name_first_page_number = "off"
			    )

# Setting the coordinates of the geographical area
map=projection["map"]


# Setting the coastlines attributes
coast = mcoast( 
			map_coastline_colour = "grey",
			map_coastline_land_shade =  "on", 
			map_coastline_land_shade_colour =  "cream", 
			map_grid =  "on", 
			map_grid_colour = "grey", 
			map_coastline_sea_shade  ="on", 
			map_coastline_sea_shade_colour=  "white", 
			map_label  ="off")

#Setting of the title
text = mtext( text_font_size = '0.7', 
		text_justification = "left",
		text_lines =  [projection["text"]])


# Plot the result
plot(output, map, coast, text)

# For documentation purposes
tofortran(projection["name"], output, map, coast, text)
tomv4(projection["name"], map)
tohtml(projection["name"],  map)



# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.


from xml.sax.handler import ContentHandler
from xml.sax import make_parser
import sys





class ObjectHandler(ContentHandler):
	name = ""
	defparam = ""
	params = []
	current = {}
	doc = ""


	def default(self, attrs):
		val = attrs.get("default");
		if (val == "-int_MAX"):
			return "-1.0E21"
		if (val == "int_MAX"):
			return "1.0E21"
		if (val == "floatarray()"):
			return "''"
		if (val == "intarray()"):
			return "''"
		if (val == "stringarray()"):
			return "''"
		if (len(val) == 0):
			return "''"
		if (val == "(automatic)"):
			return "'(automatic)'"
		return val

	def characters(self, data):
		pass
	
	def toggle(self, attrs):
		s = "\t%s\n\t{\n" % attrs.get("name")
		s = s + "\t\ton; on\n"
		s = s + "\t\toff; off\n"
		s = s + "\t} = %s\n" % self.default(attrs)
		return s


	def any(self, attrs):
		s = "DOCUMENATION " 
		s = s + "\t} = %s" % self.default(attrs)
		return s


	def number(self, attrs):
		s = "\t%s\n\t{\n" % attrs.get("name")
		s = s + "\t\t*\n"
		s = s + "\t} = %s\n" % self.default(attrs)
		return s

	def listofnumbers(self, attrs):
		s = "\t%s\n\t{\n" % attrs.get("name")
		s = s + "\t\t*\n"
		s = s + "\t\t/\n"
		s = s + "\t} = %s\n" % self.default(attrs)
		return s

	def listofstrings(self, attrs):
		s = "\t%s\n\t{\n" % attrs.get("name")
		s = s + "\t\t@\n"
		s = s + "\t\t/\n"
		s = s + "\t} = %s\n" % self.default(attrs)
		return s

	def colour(self, attrs):
		s = "\t%s\n\t{\n" % attrs.get("name")
		s = s + "\t\tred; red\n"
		s = s + "\t\t@\n"
		s = s + "\t} = %s\n" % self.default(attrs)
		return s

	def linestyle(self, attrs):
		s = "here we found a line " 
		s = s + "\t} DEFAULT %s\n" % self.default(attrs)
		return s

	def options(self, attrs):
		s = "\t%s\n\t{\n" % attrs.get("name")
		self.defparam = self.default(attrs)
		return s

	def normal(self, param):
		s =  "<tr>\n"
		s = s + "  <td><strong>%s</strong></td><td>%s</td>\n" % (param["name"], param["from"])
		s = s + "  <td>%s</td><td>%s</td>\n" % (param["doc"], param["default"])
		s = s + "</tr>\n"
		return s

	def header(self, param):
		s =  "<tr style='background-color: rgb(204, 204, 255);'>\n"
		s = s + "  <td colspan='4'><strong>%s (%s)</strong></td>\n" % (param["name"], param["value"])
		s = s + "</tr>\n"
		return s

	
	types = { "normal" : normal,
			  "header" : header}

	def newparam(self, param, value, default):
		for p in self.params:
			if p["name"] == param:
				selfcurrent = p
				return
		self.current = {"name" : param,  "type": "normal", "from" : value, "default" : default, "doc":""} 
		self.params.append(self.current)

	def startElement(self, name, attrs):
		if (name == "magics") :
			return

		if (name == "class"):
			self.name = attrs.get("name")
			self.inherits = attrs.get("inherits")
			if self.inherits != '':
				try:
					file = open("xml/%s.xml" % attrs.get("inherits"), "r")
					print " inherits ->%s" % self.inherits
					object = ObjectHandler()
					parser = make_parser()
					object.params = []
					parser.setContentHandler(object)
					parser.parse(file)
					self.params.append(s)
					for s in object.params:
						self.params.append(s)
						
				except:
					pass

		if (name == "documentation"):
			self.doc = ""
		if (name == "parameter"):
			self.param = attrs.get("name")
			print ("name %s"%self.param)
			if (attrs.get("implemented") == 'no'):
				return
			type = attrs.get("to")
			fromtype = attrs.get("from")
			deftype = attrs.get("default")
			self.newparam(self.param, fromtype, deftype)
		if (name == "option"):
				object = ObjectHandler()
				object.params = []
				object.doc = ""
				if (attrs.get("docdive") == 'no'):
					return
				parser = make_parser()
				parser.setContentHandler(object)
				file = open("xml/%s.xml" % attrs.get("name"), "r")
				value = attrs.get("fortran");
				if (attrs.get("fortran") != attrs.get("xml")) :
					value = "%s/%s"%(attrs.get("fortran"), attrs.get("xml"))
					
				p = {"name" : self.param,  "value" : value , "type" : "header"}
				self.params.append(p)
				parser.parse(file)
				for s in object.params:
					self.params.append(s)

	def characters(self, doc):
		self.doc = self.doc + doc

	def endElement(self, name):
		if (name == "magics") :
			return

			
		if (name == "documentation") :
			self.current["doc"] = self.doc
			pass
			
		if (name == "class") :
			filename = "xml/%s.html" % self.name
			# <tr style="background-color: rgb(204, 204, 255);"> 
			definition = open(filename, "w")
			definition.write( "<h3>%s parameters</h3>\n" % self.name)
			definition.write( '<table cellspacing="1" cellpadding="1" border="1" style="width: 100%;">\n')
			definition.write( '<tbody><font face="arial,sans-serif">\n')
			definition.write( "<tr>\n")
			definition.write( "  <td><strong>Parameter</strong></td><td><strong>Type</strong></td>\n")
			definition.write( "  <td><strong>Documentation</strong></td><td><strong>Default</strong></td>\n")
			definition.write( "</tr>\n")
			for param in self.params:	
				s = self.types[param["type"]](self, param)
				definition.write(s)
			definition.write( "</font></tbody>\n")
			definition.write( '</table>\n')


object = ObjectHandler()
saxparser = make_parser()
saxparser.setContentHandler(object)

datasource = open(sys.argv[1], "r")
saxparser.parse(datasource)



# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

# importing Magics module
from Magics.macro import *

ref = 'contour6'

# Setting of the output file name

output = output(output_formats=['png'],
                output_name_first_page_number='off',
                output_name=ref)


#Setting the coordinates of the geographical area
projection = mmap(subpage_upper_right_longitude = 65.00,
        		subpage_upper_right_latitude = 51.28,
		        subpage_lower_left_latitude = 21.51,
			    subpage_lower_left_longitude = -37.27,
			    subpage_map_projection = 'polar_stereographic')

coast = mcoast(map_grid='on', map_grid_colour='tan',
               map_coastline_land_shade='on',
               map_coastline_land_shade_colour='cream',
               map_coastline_colour='tan')

# Import the data
data = mgrib(grib_input_file_name='t850.grb')

# Define a contour 
list = mcont(contour_highlight='off',
			contour_level_selection_type='interval',
			contour_interval=1.,
			contour_line_colour_rainbow='on',
			contour_line_colour_rainbow_method='list',
			contour_line_colour_rainbow_colour_list=["orange", 'green', 'pink', 'navy'],
		    contour_line_colour_rainbow_colour_list_policy='cycle')

title = mtext(
           text_lines = ["<font size='1'>Rainbow technique to colour the isolines </font>",
					"using a user-defined list of colours",
		   			"<font colour='evergreen'>contour_line_colour_rainbow = on</font> ", 
					"<font colour='evergreen'>contour_line_colour_rainbow_method = list</font>"],
		   text_justification = "left",
		   text_font_size = 0.8,
           text_colour =  "charcoal")

# To the plot
plot(
    output, 
    projection,
    coast, data, list,
    title
    )


#For documentation only
tofortran(ref, output, projection,  coast, data, list, title)
tomv4(ref, list)
tohtml(ref, list)



# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.


#importing Magics module
from Magics.macro import *

#Example reference
ref = 'wind1'

#Setting of the output file name
output = output(output_formats= ['png'],
                output_name_first_page_number= 'off',
                output_name= ref)

#Setting the coordinates of the geographical area
australia = mmap(subpage_upper_right_longitude= 180.,
                 subpage_upper_right_latitude= -5.,
                 subpage_lower_left_longitude= 105.,
                 subpage_clipping='on',
                 subpage_frame_colour = "evergreen",
                             subpage_frame_line_style = "dot",
                                         subpage_frame_thickness = 5,

                 subpage_map_projection= 'cylindrical',
                 subpage_lower_left_latitude= -55.)


#Background Coastlines 
background = mcoast( map_coastline_sea_shade_colour= 'white',
                     map_coastline_land_shade_colour= 'cream',
                     map_grid= 'off',
                     map_coastline_land_shade= 'on',
                     map_coastline_sea_shade= 'on',
                     map_label= 'off',
                     map_coastline_colour= 'tan')

#Foreground Coastlines
foreground = mcoast( map_grid= 'on',
                 map_grid_colour = 'tan',
                 map_label= 'off',
                 map_coastline_colour= 'tan',
                 map_coastline_land_shade= 'off',
                 map_coastline_sea_shade= 'off')

#Import the z500 data 
msl =  mgrib( grib_input_file_name = './msl.grb',
                grib_id= 'z500')

#Define the simple contouring for msl
msl_contour = mcont(
		legend= 'off', 
		contour_level_selection_type= 'interval',
                contour_interval= 5.,
                contour_line_colour= 'black',
                contour_line_thickness= 1,
                contour_hilo= 'on',
                contour_hilo_quality= 'high',
                contour_hi_colour= 'black',
                contour_lo_colour= 'black',
                contour_hilo_height= 0.25,
                contour_label= 'off',
                contour_highlight_colour= 'black',
                contour_highlight_thickness= 2,
                )

#Import the  wind speed at 200hPa speed200 
speed200 =  mgrib( grib_input_file_name = './speed200.grb',
                grib_id= 't850')

#Define the shading for the wind speed
speed200_contour = mcont(
    legend= 'on',
    contour_level_selection_type= 'level_list', 
    contour_level_list= [30., 40., 50., 60., 70., 80., 90., 100.], 
    contour_shade= 'on', 
    contour_shade_max_level_colour= 'evergreen', 
    contour_shade_min_level_colour= 'yellow',
    contour_shade_method= 'area_fill', 
    contour_reference_level= 0., 
    contour_highlight= 'off', 
    contour_hilo= 'on', 
    contour_hilo_format= '(F3.0)', 
    contour_hilo_height= 0.2, 
    contour_hilo_type= 'number', 
    contour_hilo_suppress_radius= 30., 
    contour_hi_min_value= 15., 
    contour_label= 'off' 
         )


#Import the  wind  at 200hPa uv200 
uv200 =  mgrib( grib_input_file_name = './uv200.grb',
                grib_id= 'uv200')

uv200_wind = mwind(
                legend= 'on',
                wind_field_type= 'arrows',
                wind_arrow_thickness= 1,
                wind_thinning_factor= 3.,
                wind_arrow_colour= 'gold' )

title = mtext(text_lines = ["<font size='1'>Wind arrows with legend</font>",
	                    "",
	                    "<grib_info id='z500' key='name' format='%s'/>"],
	      text_justification = 'left',
	      text_font_size = 0.5,
	      text_colour = 'charcoal')

#add a legend
legend = mlegend(legend= 'on',
           legend_text_colour= 'black',
           legend_box_mode= 'positional',
           legend_box_x_position= 27.5,
           legend_box_y_position= 4.,
           legend_box_x_length= 2.,
           legend_box_y_length= 12.,
           legend_border= 'off',
           legend_border_colour= 'black',
           legend_box_blanking= 'on',
           legend_display_type= 'continuous',
           legend_title = "on",
	   legend_title_text= "Wind speed at 200 hPa",
	   legend_text_font_size = "0.5")

#To the plot
plot(output, australia, background,speed200, speed200_contour,uv200, uv200_wind,msl, msl_contour,foreground, title, legend)

#For documentation only
tofortran(ref, output, australia, background,speed200, speed200_contour,uv200, uv200_wind,msl, msl_contour,foreground, title, legend)
tomv4(ref,uv200_wind)
tohtml(ref,uv200_wind)



# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

#importing Magics module
from Magics.macro import *


ref = 'coastlines4'
#Setting of the output file name
output = output(output_formats = ['png'],output_name_first_page_number = "off",output_name = ref)

#Setting the coordinates of the geographical area
projection = mmap(subpage_map_projection = 'cylindrical',
		  subpage_lower_left_latitude= 34.,
		  subpage_lower_left_longitude= -20.,
		  subpage_upper_right_latitude= 60.,
		  subpage_upper_right_longitude= 16.)

#Coastlines setting
coast = mcoast( map_grid = "off",
		map_coastline_resolution = "high",
		map_coastline_land_shade  =  "off",
		map_coastline_colour = "sky",
		map_coastline_sea_shade = "on",
		map_coastline_sea_shade_colour = "sky",
		map_rivers = "on",
		map_rivers_colour = "sky")

title = mtext(
	  text_lines = ["<font size='1'>Sea, lakes and rivers</font>"],
	  text_justification = "left",
	  text_font_size = 0.8,
	  text_colour = "charcoal")

#To the plot
plot(output, projection, coast,title)

#For documentation only
tofortran(ref, output, projection, coast, title)
tomv4(ref,coast)
tohtml(ref,coast)
















# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.


#importing Magics module
from Magics.macro import *


ref = 'contour19'
#Setting of the output file name
output = output(output_formats = ['png'], 
		output_name_first_page_number = "off",
		output_name = ref)

#Setting the coordinates of the geographical area
projection = mmap(
    subpage_map_projection='cartesian',
    subpage_x_axis_type='regular',
    subpage_x_automatic='on',
    subpage_y_axis_type='regular',
    subpage_y_automatic='on',
    subpage_y_position=3.,
    )


horizontal = maxis(
    axis_orientation='horizontal',
    axis_grid='on',
    axis_grid_colour='Rgb(0.5, 0.5, 0.5)',
    axis_grid_line_style='solid',
    axis_line_colour='grey',
    axis_minor_tick='on',
    axis_minor_tick_colour='grey',
    axis_tick_colour='grey',
    )

# define vertical axis

vertical = maxis(
    axis_orientation='vertical',
    axis_grid='on',
    axis_grid_colour='Rgb(0.5, 0.5, 0.5)',
    axis_grid_line_style='dash',
    axis_line='on',
    axis_line_colour='grey',
    axis_tick='on',
    axis_tick_label='on',
    )



#Import the data 
data =  mnetcdf(netcdf_type = "matrix",
    netcdf_y_variable = "latitude",
    netcdf_x_variable = "longitude",
    netcdf_value_variable = "u10",
  netcdf_filename = "era5_2000_aug_1.nc")


#Define the simple contouring for z500
contour = mcont( legend = "off",
                contour_line_colour = "navy",
                contour_line_thickness =  2,
                contour_label =  "on",
                contour_highlight_colour =  "navy",
                contour_highlight_thickness =  6 )





title = mtext(
           text_lines = ["<font size='1'>Loading NetCDF...</font>",
		   			"<font colour='evergreen'>contour_level_selection_type = count</font> ", 
					"    calculate a reasonable  contour interval from the min and max of the displayed field"],
		   text_justification = "left",
		   text_font_size = 0.8,
           text_colour =  "charcoal")


#To the plot
plot(output, projection, horizontal, vertical, data, contour, title)


















# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

#importing Magics module
from Magics.macro import *


ref = 'coastlines5'
#Setting of the output file name
output = output(output_formats = ['png'],output_name_first_page_number = "off",output_name = ref)

#Setting the coordinates of the geographical area
projection = mmap(subpage_map_projection = 'polar_stereographic')

#Coastlines setting
coast = mcoast( map_grid =  "on",
                map_grid_colour  =  "tan",
		map_coastline_land_shade  =  'on',
		map_coastline_land_shade_colour  =  'cream',
		map_coastline_sea_shade  =  'on',
		map_coastline_sea_shade_colour  =  'blue',
		map_coastline_colour =  "tan",
		map_coastline_style = "dash",
        map_label_font= "times",
        map_label_font_style= "italic",
        map_label_colour= "evergreen",
        map_label_blanking= "off",
        map_label_height= 0.6,
        )


#Title settings
title = mtext(
	  text_lines = ["<font size='1'>Test map_label attributes</font>"],
	  text_justification = "left",
	  text_font_size = 0.8,
	  text_colour = "charcoal")

#To the plot
plot(output, projection, coast,title)


















# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.


#importing Magics module
from Magics.macro import *


ref = 'graph2'
#Setting of the output file name
output = output(output_formats = ['png'], 
		output_name_first_page_number = "off",
		output_name = ref)

#Setting the cartesian view
projection = mmap( subpage_y_position= 2., 
			subpage_map_projection = 'cartesian',
			subpage_x_axis_type = 'date',
			subpage_y_axis_type = 'regular',
			subpage_x_date_min = "2012-03-01 12:00:00",
			subpage_x_date_max = "2012-03-03 12:00:00",
			subpage_y_min = 25.,
			subpage_y_max = 75.)

#Vertical axis
vertical = maxis(axis_orientation = "vertical",
				 axis_type = "regular",
			     axis_tick_label_height = 0.4,
				 axis_tick_label_colour = 'navy',
				 axis_grid =  "on",
				 axis_grid_colour = "grey",
				 axis_grid_thickness = 1,
				 axis_grid_line_style = "dot")

#Horizontal axis
horizontal = maxis(axis_orientation = "horizontal",
				 axis_type = "date",
				 axis_grid =  "on",
				 axis_days_label_height = 0.40,
			 	 axis_months_label_height = 0.40,
				 axis_years_label_height = 0.50,
				 axis_grid_colour = "grey",
				 axis_grid_thickness = 1,
				 axis_grid_line_style = "dot")



#define  the  data 
x =  ["2012-03-02 00:00:00","2012-03-02 12:00:00","2012-03-03 00:00:00"]
y = numpy.array([50.,30.,50.])

input = minput(input_y_values=y,
			input_date_x_values= x,)
#Define the graph 
graph = mgraph( 
            graph_type="bar",
			graph_bar_colour='evergreen',
			graph_bar_width=7200. )



title = mtext(
           text_lines = ["Simple Bar"],
		   text_justification = "left",
		   text_font_size = 1.,
           text_colour =  "charcoal")


#To the plot
plot(output, projection, vertical, horizontal, input, graph, title)

#For documentation only
tofortran(ref, output, projection, vertical, horizontal, input, graph, title)
tomv4(ref, graph)
tohtml(ref, input, graph)

















# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.


#importing Magics module
from Magics.macro import *

#Example reference
ref = 'wind4'

#Setting of the output file name
output = output(output_formats= ['png'],
                output_name_first_page_number= 'off',
                output_name= ref)


#Setting the coordinates of the geographical area
europe = mmap( subpage_lower_left_latitude= 21.51,
               subpage_lower_left_longitude= -37.27,
               subpage_upper_right_latitude= 51.28,
               subpage_upper_right_longitude= 65.0,
               subpage_map_projection= "polar_stereographic")

#Background Coastlines 
foreground = mcoast({"map_coastline_colour": "black",
                     "map_coastline_land_shade": "off",
                     "map_coastline_sea_shade": "off",
                     "map_grid": "off",
                     "map_label": "off"})


#Import the  wind
uv700 =  mgrib( grib_input_file_name = './wind.grib',grib_id= 'uv700')

uv700_wind = mwind(wind_arrow_unit_system= "automatic",
                   wind_arrow_unit_velocity= 10.0,
                   wind_field_type= "Flags",
                   wind_flag_colour= "black",
                   wind_flag_length= 0.3,
                   wind_flag_origin_marker= "dot",
                   wind_flag_origin_marker_size= 0.1,
                   wind_legend_text= "Black wind flags (in knots)",
                   wind_thinning_factor= 1.,
                   wind_thinning_method= "automatic")

title = mtext( text_lines = ["<font size='1'>Vorticity and wind flags</font>",'Vorticity at 700 hPa'],
		text_mode= 'positional',
		text_box_x_position= 1.0,
		text_box_y_position= 17.5,
		text_box_x_length= 27.0,
		text_box_y_length= 2.3,
	       text_justification = 'center',
	       text_font_size = 0.5,
	       text_colour = 'charcoal')

#Import the  vorticity
vo700 =  mgrib( grib_input_file_name = './vorticity.grib',grib_id= 'vo700')

#vorticity contour
vo700_contour = mcont({
                    "legend":"on", 
                    "contour": "off",
                    "contour_highlight": "off",
                    "contour_hilo": "off",
                    "contour_label": "off",
                    "contour_legend_text": "Contour shade (MORE CONTOURS) Range: -200 / 200",
                    "contour_level_list": [-200.0, -100.0, -75.0, -50.0, -30.0, -20.0, -15.0, -13.0, -11.0, -9.0, -7.0, -5.0, -3.0, -1.0, 1.0, 3.0, 5.0, 7.0, 9.0, 11.0, 13.0, 15.0, 20.0, 30.0, 50.0, 75.0, 100.0, 200.0],
                    "contour_level_selection_type": "level_list",
                    "contour_shade": "on",
                    "contour_shade_colour_list": ["rgb(0,0,0.3)","rgb(0,0,0.5)","rgb(0,0,0.7)","rgb(0,0,0.9)","rgb(0,0.15,1)","rgb(0,0.3,1)","rgb(0,0.45,1)","rgb(0,0.6,1)","rgb(0,0.75,1)","rgb(0,0.85,1)","rgb(0.2,0.95,1)","rgb(0.45,1,1)","rgb(0.75,1,1)","none","rgb(1,1,0)","rgb(1,0.9,0)","rgb(1,0.8,0)","rgb(1,0.7,0)","rgb(1,0.6,0)","rgb(1,0.5,0)","rgb(1,0.4,0)","rgb(1,0.3,0)","rgb(1,0.15,0)","rgb(0.9,0,0)","rgb(0.7,0,0)","rgb(0.5,0,0)","rgb(0.3,0,0)"],
                    "contour_shade_colour_method": "list",
                    "contour_shade_method": "area_fill"}) 

vo700_legend = mlegend(legend= 'on',
           legend_box_mode= 'positional',
           legend_box_x_position= 1.0,
           legend_box_y_position= 16.0,
           legend_box_x_length= 27.0,
           legend_box_y_length= 1.5,
           legend_text_colour= 'black',
           legend_border= 'off',
           legend_border_colour= 'black',
           legend_box_blanking= 'on',
           legend_display_type= 'continuous',
	   legend_text_font_size = 0.5)

#To the plot
plot(output,europe,vo700,vo700_contour,vo700_legend,uv700,uv700_wind,foreground,title)

#For documentation only
tofortran(ref,output,europe,vo700,vo700_contour,vo700_legend,uv700,uv700_wind,foreground,title)
tomv4(ref,uv700_wind)
tohtml(ref,uv700_wind)



# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.


#importing Magics module
from Magics.macro import *

#Example reference
ref = 'legend3'

#Setting of the output file name
output = output(output_formats= ['png'],
                output_name_first_page_number= "off",
                output_name= ref)

#Setting the coordinates of the geographical area
europe = mmap(subpage_upper_right_longitude= 65.,
              subpage_map_projection= "polar_stereographic",
              subpage_map_vertical_longitude= 0.0,
              subpage_lower_left_longitude= -37.27,
              subpage_lower_left_latitude= 21.51,
              subpage_upper_right_latitude= 51.28)

#Coastlines setting
coast = mcoast( map_grid= "on",
                map_grid_colour= "tan",
                map_coastline_colour= "tan",
                map_coastline_resolution= "high")

#Import the z500 data 
z500 =  mgrib(grib_input_file_name = "./z500.grb",
              grib_id= "z500")

#Define the simple contouring for z500
z500_contour = mcont(legend= "off",
                     contour_level_selection_type= "interval",
                     contour_line_colour= "black",
                     contour_hilo_height= 0.25,
                     contour_line_thickness= 1,
                     contour_hilo= "on",
                     contour_hilo_quality= "high",
                     contour_label= "off",
                     contour_highlight_colour= "black",
                     contour_highlight_thickness= 2,
                     contour_interval= 5.)

#Import the t850 data 
t850 =  mgrib( grib_input_file_name = "./t850.grb",
               grib_id= "t850")


#Define the shading for t850
t850_contour = mcont(legend="on",
		     contour_shade= "on",
		     contour_hilo= "off",
		     contour= "off",
		     contour_label= "off",
		     contour_shade_method= "area_fill",
		     contour_level_selection_type = "level_list",
		     contour_level_list = [-10.0,-8.0,-6.0,-4.0,-2.0,0.0,2.0,4.0,6.0,8.0,10.0],
		     contour_shade_max_level_colour= "red",
		     contour_shade_min_level_colour= "blue",
		     contour_shade_colour_direction= "clockwise")

    


title = mtext(text_lines = ["<font size='1'>Positional and histogram legend</font>","","","","",""],
	      text_justification = "left",
	      text_font_size = 0.5,
	      text_colour = "charcoal")

#add a legend
legend = mlegend(legend = "on",
 		 legend_text_colour="black",
		 legend_box_mode= "positional",
		 legend_box_x_position= 1.0,
		 legend_box_y_position= 16.0,
		 legend_box_x_length= 27.,
		 legend_box_y_length= 3., 		 
		 legend_display_type = "histogram",
		 legend_label_frequency = 1,
		 legend_histogram_mean_value = "on",
		 legend_title = "on",
		 legend_title_text= "Temperature at 850 hPa",
		 legend_text_font_size = 0.5)

#To the plot
plot(output, europe, t850, t850_contour, z500, z500_contour, coast, title, legend)

#For documentation only
tofortran(ref, output, europe, t850, t850_contour, z500, z500_contour, coast, title, legend)
tomv4(ref,legend)
tohtml(ref,legend)




# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

# importing Magics module

from Magics.macro import *

ref = 'contour8'

# Setting of the output file name

output = output(output_formats=['png'],
                output_name_first_page_number='off',
                output_name=ref)

# Setting the coordinates of the geographical area

projection = mmap(subpage_upper_right_longitude=180.00,
                  subpage_upper_right_latitude=90.00,
                  subpage_lower_left_latitude=-90.00,
                  subpage_lower_left_longitude=-180.0,
                  subpage_map_projection='cylindrical')



coast = mcoast(map_grid='on', map_grid_colour='tan',
               map_coastline_land_shade='on',
               map_coastline_land_shade_colour='cream',
               map_coastline_colour='tan')

# Import the data
data = mgrib(grib_input_file_name='Total_precipitation.grib')

# Define a contour 
ecchart = mcont(contour_automatic_setting='ecchart',)



# Define a title 
title = mtext(
           text_lines = ["<font size='1'>Choose eccharts styling depending on the parameter</font>",
					"Here: ecchart for parameter <grib_info key='name'/> at  <grib_info key='level'/> hPa",
		   			"<font colour='evergreen'>contour_automatic_setting=ecchart</font> ",],
		   text_justification = "left",
		   text_font_size = 0.8,
           text_colour =  "charcoal")

# To the plot

plot(
    output, projection, coast,
	data, ecchart, title)

#For documentation only
tofortran(ref, output,  projection, coast, data, ecchart, title)
tomv4(ref, ecchart)
tohtml(ref, ecchart)




# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.


#importing Magics module
from Magics.macro import *


ref = 'contour20'
#Setting of the output file name
output = output(output_formats = ['png'], 
		output_name_first_page_number = "off",
		output_name = ref)

#Setting the coordinates of the geographical area
projection = mmap(subpage_map_projection = 'mercator')


#Coastlines setting
coast = mcoast( map_grid =  "on",
                map_grid_colour  =  "tan",
				map_coastline_land_shade  =  'on',
				map_coastline_land_shade_colour  =  'cream',
				map_coastline_colour =  "tan")


#Import the data 
data =  mnetcdf(netcdf_type = "geomatrix",
    netcdf_x_variable = "latitude",
    netcdf_y_variable = "longitude",
    netcdf_x_component_variable = "u10",
    netcdf_y_component_variable = "v10",
  netcdf_filename = "era5_2000_aug_1.nc")


#Define the simple contouring for z500
wind = mwind( legend = "off",
                )





title = mtext(
           text_lines = ["<font size='1'>Loading NetCDF...</font>",
		   			"<font colour='evergreen'>contour_level_selection_type = count</font> ", 
					"    calculate a reasonable  contour interval from the min and max of the displayed field"],
		   text_justification = "left",
		   text_font_size = 0.8,
           text_colour =  "charcoal")


#To the plot
plot(output, projection, coast, data, wind, title)


















#!/usr/bin/env python 
# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

from xml.sax.handler import ContentHandler
from xml.sax import make_parser
from types import *

import sys
import json


class ObjectHandler(ContentHandler):
	path ="../src/xml/"
	tag = ""
	doc = ""
	magics = {}
	objects = {}
	object = {}
	param = {}
	paramdoc = ""
	actiondoc = ""

	def boolset(p, attrs):
		p["values"] = ["on", "off"]

	def lineset(p, attrs):
		p["values"] = ["solid", "dash", "dot"]
		p["type"] = "toggle"
		
	def anyset(p, attrs):
		if  attrs.has_key("values") :
			val = attrs["values"]
			values = val.split("/")
			p["values"] = values
			p["type"] = "toggle"
		return
	
	types = { 
		"bool": boolset,
		"string": anyset,
		"Colour": anyset,
		"LineStyle": lineset,
	}


	def get(self, attrs, name, default):
		if  attrs.has_key(name):
			return attrs.get(name)
		else :
			return default

	def newparam(self, attrs):
		p = {}
		p["name"] = attrs.get("name")
		p["type"] = attrs.get("to")
		p["values"] = []
		t = p["type"]
		if  self.types.has_key(t) :
			self.types[t](p, attrs)
		else :
			p["type"]="toggle"



		p["default"] = attrs.get("default")
		return p

	def newobject(self, attrs):
		o = {}
		o["name"] = attrs.get("name")
		o["documentation"] = ""
		o["parameters"] = []
		if  attrs.has_key("inherits") :
			o["inherits"]=attrs.get("inherits")
			print o["name"] + " inhrits from " + o["inherits"]
		return o

	def startElement(self, name, attrs):
		self.tag = name
		if ( name == "class" ) : 
			self.doc = name
		# New object
			self.object = self.newobject(attrs)
			self.magics[attrs.get("name")] = self.object;
		if ( name == "parameter" ) : 
			if  self.get(attrs, "visible", "on") == "on" :
				self.doc = name
				self.param = self.newparam(attrs)
				self.object["parameters"].append(self.param)
			else :
				self.doc = "nodoc"
		if ( name == "option") : 
			val = attrs.get("fortran")
			self.param["values"].append(val)
			if ( self.param.has_key(val) == False ):
				self.param[val] = []
			self.param[val].append(attrs.get("name"))
		if ( name == "set" ) : 
			val = attrs.get("value")
			if ( self.param.has_key(val) == False ):
				self.param[val] = []
			self.param[val].append(attrs.get("name"))

		

	def characters(self, content):
		if ( self.tag == "documentation" ) : 
			if ( self.doc == "class" ) :
				self.actiondoc += content.lstrip();
			if ( self.doc == "parameter" ) :
				self.paramdoc += content.lstrip();
				

	def endElement(self, name):
		if ( name == "parameter"  and self.doc == "parameter") : 
			self.param["documentation"] = self.paramdoc
			self.paramdoc= ""
		if ( name == "class" ) : 
			self.object["documentation"] = self.actiondoc
			self.actiondoc= ""
			
	def printDef(self):
		print json.dumps(self.magics, indent=2)
	

	def parse(self, file):
		saxparser = make_parser()
		saxparser.setContentHandler(self)
		datasource = open(self.path + file, "r")
		saxparser.parse(datasource)
		return self.magics
	
	def build(self, actions, args):
		defs = args["parameters"]
		parameters = []
			
		for a in actions:
			if ( type(a) != dict and self.magics.has_key(a) ) :
			# external object
				action = self.magics[a]
				if action.has_key("inherits") :
					parent = action["inherits"]
					definition = self.magics[parent]
					print  a + " xxxxinherits from " + parent
			
					print "??????????????????"
					for p in definition["parameters"]:
						print p["name"]	
						parameters.append(p)
					print "---------------"

				for param in action["parameters"]:
					#print "add" + param["name"]
					parameters.append(param)
					for val in param["values"]:
						if param.has_key(val) :
							param[val] = self.build(param[val], action)
						else :
							param[val] = [] 
			else :
				for param in defs:
					if (param["name"] == a) :
						parameters.append(param)
						defs.remove(param)
					

		return parameters;

	def getParams(self, action):
		return self.build(action, self.magics[action[0]]);

	def getDoc(self, action):
		action = self.magics[action]

		return action["documentation"]
			


object = ObjectHandler()
object.parse("Coastlines.xml")
object.parse("CoastPlotting.xml")
object.parse("LabelPlotting.xml")
object.parse("GridPlotting.xml")
parameters = object.getParams(["Coastlines"]);
doc = object.getDoc("Coastlines");

magics={}
magics["magics"] = []
action={}
action["action"]="coastlines"
action["documentation"]=doc
action["parameters"]=parameters
magics["magics"].append(action)



f = open('coast.json', "w")
f.write(json.dumps(magics, indent=1))




#!/usr/bin/env python
# (C) Copyright 1996-2016 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

from xml.sax.handler import ContentHandler
from xml.sax import make_parser
from types import *

import sys


if(len(sys.argv) != 6) :
    print("\n\tYou need to give 4 input parameters:")
    print("\n\t  %s source.xml targetDef CLASS_NAME rulesDef\n",sys.argv[0])
    sys.exit()



# ObjectHandler::optionalparams
# - will store dictionary with optional param name as key, and list of tuples as value
# - each tuple is a pair: (conditional_param_name, value-to-enable)
# e.g.
# {"symbol_text_font_name": (("symbol_type", "text"), ("symbol_type", "both"))}
# - which means that if "symbol_type" is either "text" or "both", then "symbol_text_font_name"
#   should be enabled; or in Metview rules terms, if "symbol_type is neither"
#   "text" nor "both", then "symbol_text_font_name" should be disabled.
#   NOTE: we make an assumption - if two conditional parameters are the same (e.g.
#   "symbol_type" in the above example), then the relationship is %or, otherwise it
#   is %and. This should be the case because of the way the .xml files are arranged.

# ObjectHandler::optionalparams2
# - a reorganised version of ObjectHandler::optionalparams
# - this time, the keys are the tuples of conditions and the values are the
#   set of parameters that are to be set. This is so that we can combine all
#   the %unset parameters together under the same condition.


# ObjectHandler::optionalparams3
# - an ordered version of optionalparams2 using a tuple instead of a dictionary (so that we
#   can ensure a 'working' order of the conditions, ie make it impossible to unset a parameter
#   before it needs to be queried, because this causes an error in applying the rules).
#   (  ((condition1,condition2), (unsetparam1,unsetparam2)),  ((condition3,condition4), (unsetparam3,unsetparam4)) )


# ObjectHandler::classes
# - stores all the parsed class names along with their associated parameters, e.g.
# {"MarkerShadingTechnique": {"inherits": ["ShadingTechnique"],
#                             "params": ["contour_shade_colour_table", "contour_shade_height_table"]}}

class ObjectHandler(ContentHandler):
	title = ""
	classname = ""
	defparam = ""
	tab1 = 0
	tab2 = 2
	ignore = False
	mydef = []   # the growing list of parameters
	param = ""
	myoptions = []
	optionalparams  = {}
	optionalparams2 = {}  # see reorganiseOptionalParameters()
	optionalparams3 = ()
	myrules = {}
	objects = []
	last = ""   # the currently-parsed parameter
	classes = {}
	#inheritedlast = ""
	unset = {}
	filehistory = []  # a list of files we have opened (so that we don't open the same one twice)
	debug = 0  # gives some output on stdout
	toplevel = True

	def setTopLevel(self, top):
		self.toplevel = top


	def default(self, attrs):
		val = attrs.get("metview_default");
		if val is None:
			val = attrs.get("default");

		if (val == "") :
			val = attrs.get("default");
		if (val == ".") :
			val = "'.'";
		if (val.upper() == "-INT_MAX"):
			return "-1.0E21"
		if (val.upper() == "INT_MAX"):
			return "1.0E21"
		if (val.upper() == "-LLONG_MAX"):
			return "-1.0E21"
		if (val.upper() == "LLONG_MAX"):
			return "1.0E21"
		if (val == "floatarray()"):
			return "''"
		if (val == "intarray()"):
			return "''"
		if (val == "stringarray()"):
			return "''"
		if (len(val) == 0):
			return "''"
		if (val == "(automatic)"):
			return "'(AUTOMATIC)'"
		return val

	def tab(self, n):
		sep = ''
		i = 0
		while i < n:
			sep = sep + '\t'
			i = i + 1
		return sep

	def nice(self, name, attrs):
		sep1 = self.tab(self.tab1)
		sep2 = self.tab(self.tab2)
		self.file.write("%s<%s " % (sep1.upper(), name.upper()))
		for key in attrs.getNames():
			self.file.write(" %s='%s'" % (key.upper() , attrs.getValue(key).upper()))
		self.file.write(">\n")

	def characters(self, data):
		pass


	# addhidden - if the appropriate flag is set in the attributes, return the text that will
	#             make the parameter hidden (but available) in Metview
	def addhidden(self, attrs):
		if attrs.get("metview") == 'hidden':
			return "\t[ visible = false ]\n"
		else:
			return ""

	def toggle(self, attrs):
		s = "\t%s\n%s\t{\n" % (attrs.get("name").upper(), self.addhidden(attrs))
		s = s + "\t\tON;  ON\n"
		s = s + "\t\tOFF; OFF\n"
		s = s + "\t} = %s\n" % self.default(attrs).upper()
		return s


	def any(self, attrs):
		v = attrs.get("values")
		if  v != None:
			return	self.list(attrs)
		x = -1
		for key in self.predefined_list:
			x = self.param.find(key)
			if x != -1 :
				f = self.predefined[key]
				# only use the predefined function if the seach string
				# occurs at the end of the parameter name
				if x + len(key) == len(self.param):
					self.newparam(self.param, f(self, attrs), self.default(attrs))
					return
		s = "\t%s\n%s\t{\n" % (attrs.get("name").upper(), self.addhidden(attrs))
		s = s + "\t\t@\n"
		default_value = self.default(attrs)
		# special case: html tags seem to have their quotes removed, so we need to put them back
		if len(default_value) > 0 and default_value[0] == "<":
			default_value = "'" + default_value + "'"
		else:
			default_value = default_value.upper()
		s = s + "\t} = %s\n" % default_value
		return s


	def number(self, attrs):
		s = "\t%s\n%s\t{\n" % (attrs.get("name").upper(), self.addhidden(attrs))
		s = s + "\t\t*\n"
		s = s + "\t} = %s\n" % self.default(attrs).upper()
		return s

	def listofnumbers(self, attrs):
		if (attrs.get("name").upper() == "OUTPUT_FRAME_LIST"):
			s = "\n"
			return s
		s = "\t%s\n%s\t{\n" % (attrs.get("name").upper(), self.addhidden(attrs))
		s = s + "\t\t*\n"
		s = s + "\t\t/\n"
		s = s + "\t} = %s\n" % self.default(attrs).upper()
		return s

	def listofstrings(self, attrs):
		if (attrs.get("colourlist") == "on" ) :
			 s = "\t%s [ help = help_colour,interface = colour ]\n%s\t{ \n" % (attrs.get("name").upper(), self.addhidden(attrs))
			 s = s + "\t\t&PARAMSHARE&COLOUR"
			 s = s + "\t\t/\n"
			 s = s + "\t} = %s\n" % self.default(attrs).upper()
		elif (attrs.get("countrylist") == "on" ):
			s = "\t%s [ help = help_multiple_selection, exclusive = False ]\n%s\t{\n" % (attrs.get("name").upper(),self.addhidden(attrs) )
			countries = attrs.get("values").split("/")
			for c in countries:
				parts = c.split(':')
				shortname = parts[0]
				longname = parts[1].replace(',', ' ') # cannot have special characters here
				longname = longname.replace('(', ' ')
				longname = longname.replace(')', ' ')
				s = s + "\t\t%s;%s\n" %(longname.upper(), shortname.upper())
			s = s + "\t\t*\n"
			s = s + "\t\t/\n"
			s = s + "\t} = %s\n" % self.default(attrs).upper()
		else:
			s = "\t%s\n%s\t{\n" % (attrs.get("name").upper(), self.addhidden(attrs))
			s = s + "\t\t@\n"
			s = s + "\t\t/\n"
			s = s + "\t} = %s\n" % self.default(attrs).upper()

		return s


	def colour(self, attrs):
		s = "\t%s [ help = help_colour,interface = colour ]\n%s\t{ \n" % (attrs.get("name").upper(), self.addhidden(attrs))
		s = s + "\t\t&PARAMSHARE&COLOUR\n"
		mi = attrs.get("metview_interface")
		if mi == "ColourWithExtra":
			s = s + "\t\t%s\n" % self.default(attrs).upper()
		s = s + "\t} = %s\n" % self.default(attrs).upper()
		return s

	def colourtechnique(self, attrs):
		s = "\t%s\n%s\t{\n" % (attrs.get("name").upper(), self.addhidden(attrs))
		s = s + "\t\tCALCULATE;CALCULATE\n"
		s = s + "\t\tLIST;LIST\n"
		s = s + "\t} = %s\n" % self.default(attrs).upper()
		return s.upper()

	def linestyle(self, attrs):
		s = "\t%s\n%s\t{\n" % (attrs.get("name").upper(), self.addhidden(attrs))
		s = s + "\t\t&PARAMSHARE&STYLE\n"
		s = s + "\t} = %s\n" % self.default(attrs).upper()
		return s

	def policy(self, attrs):
		s = "\t%s\n%s\t{\n" % (attrs.get("name"), self.addhidden(attrs))
		s = s + "\t\tlastone;lastone\n"
		s = s + "\t\tcycle;cycle\n"
		s = s + "\t} = %s\n" % self.default(attrs)
		return s.upper()

	def fontstyle(self, attrs):
		s = "\t%s\n%s\t{\n" % (attrs.get("name").upper(), self.addhidden(attrs))
		s = s + "\t\tnormal;normal\n"
		s = s + "\t\tbold;bold\n"
		s = s + "\t\titalic;italic\n"
		s = s + "\t\tbolditalic;bolditalic\n"
		s = s + "\t} = %s\n" % self.default(attrs).upper()
		return s.upper()

	def justification(self, attrs):
		s = "\t%s\n%s\t{\n" % (attrs.get("name").upper(), self.addhidden(attrs))
		s = s + "\t\tleft;left\n"
		s = s + "\t\tcentre;centre\n"
		s = s + "\t\tright;right\n"
		s = s + "\t} = %s\n" % self.default(attrs).upper()

		return s.upper()
	def font(self, attrs):
		s = "\t%s\n%s\t{\n" % (attrs.get("name").upper(), self.addhidden(attrs))
		s = s + "\t\t&PARAMSHARE&FONT\n"
		s = s + "\t} = %s\n" % self.default(attrs).upper()
		return s.upper()

	def quality(self, attrs):
		s = "\t%s\n%s\t{\n" % (attrs.get("name"), self.addhidden(attrs))
		s = s + "\t\thigh;high\n"
		s = s + "\t\tmedium;medium\n"
		s = s + "\t\tlow;low\n"
		s = s + "\t} = %s\n" % self.default(attrs)
		return s.upper()

	def list(self, attrs):
		s = "\t%s\n%s\t{\n" % (attrs.get("name").upper(), self.addhidden(attrs))
		keys = attrs.get("values").split("/")
		for key in keys:
			s = s + "\t\t%s;%s\n" %(key.upper(), key.upper())
		s = s + "\t} = %s\n" % self.default(attrs).upper()
		return s

	def options(self, attrs):
		s = "\t%s\n%s\t{\n" % (attrs.get("name").upper(), self.addhidden(attrs))
		self.defparam = self.default(attrs).upper()
		return s

	predefined = {"font_style" : fontstyle,
			  "font": font,
			  "quality": quality
			  }

	predefined_list = ["font_style", "font", "quality" ]

	types = { "bool"            : toggle,
			  "string"          : any,
			  "Path"            : any,
			  "Position"        : any,
			  "int"             : number,
			  "Colour"          : colour,
			  "ColourWithExtra" : colour,
			  "ColourTechnique" : colourtechnique,
			  "LineStyle"       : linestyle,
			  "Justification"   : justification,
			  "ListPolicy"      : policy,
			  "floatarray"      : listofnumbers,
			  "intarray"        : listofnumbers,
			  "stringarray"     : listofstrings,
			  "float"           : number}

	def newparam(self, param, value, default):
		#print('  newparam: ', param, ' ', value, ' ', default)
		for p in self.mydef:
			if p[0] == param:
				return
		self.mydef.append((param, value, default))


	def addOptionalParam(self, paramname, requirements):
		if paramname in self.optionalparams.keys():
			if not(requirements[0] in self.optionalparams[paramname]):
				self.optionalparams[paramname] = self.optionalparams[paramname] + requirements
		else:
			self.optionalparams[paramname] = requirements


	def addRelatedOptionalRequirements(self):
		optparams = self.optionalparams.keys()
		for optparam in optparams:
			conditions = self.optionalparams[optparam]
			for condition in conditions:
				if condition[0] in optparams:
					self.addOptionalParam(optparam, (condition,))


	def addRelatedOptionalRequirements2(self):
		optparams = self.optionalparams.keys()
		for optparam in optparams:
			conditions = self.optionalparams[optparam]
			for condition in conditions:
				if condition[0] in optparams:
					addreqs = self.optionalparams[condition[0]]
					self.addOptionalParam(optparam, addreqs)




	# see top of file for description of optionalparams and optionalparams2
	def reorganiseOptionalParameters(self):
		for optparam in self.optionalparams.keys():
			condition = self.optionalparams[optparam]
			if condition in self.optionalparams2.keys():
				self.optionalparams2[condition].add(optparam)
			else:
				self.optionalparams2[condition] = set([optparam])


	# put the dictionary in optionalparams2 into tuple in optionalparams3 like this:
	#   (  ((condition1,condition2), (unsetparam1,unsetparam2)),  ((condition3,condition4), (unsetparam3,unsetparam4)) )
	# see comments at top of file
	def ensureOptionalParameters2DependenciesAreInRightOrder(self):
		self.optionalparams3 = () # empty tuple, will contain an ordered version of optionalparams2

		for conditions2 in self.optionalparams2.keys():
			# does this rule unset something that is needed in the optionalparams3 rules?
			unsetparams2 = self.optionalparams2[conditions2]
			newtuple3 = (conditions2, unsetparams2)
			position = 0
			foundconflict = False
			for clause3 in self.optionalparams3:  # look through the new set of rules
				unsetparams3 = clause3[1]
				for enquireparam2 in conditions2:
					if enquireparam2[0] in unsetparams3:  # this clause2 should go *after* the current clause3
						foundconflict = True
						break
				if foundconflict:
					break

				position = position + 1

			# we now know that the clause should go into position 'position' in the new tuple
			if self.debug:
				print("put into position " + str(position))
			if position == 0:
				firstpart = ()
				secondpart = (newtuple3,)
				lastpart = self.optionalparams3
			else:
				if position == len(self.optionalparams3):
					firstpart = self.optionalparams3
					secondpart = (newtuple3,)
					lastpart = ()
				else:
					firstpart = self.optionalparams3[0:position]
					secondpart = (newtuple3,)
					lastpart = self.optionalparams3[position:len(self.optionalparams3)]

			self.optionalparams3 = firstpart + secondpart + lastpart




	def newClass(self, c):
		if not (c in self.classes.keys()):
			self.classes[c] = {"inherits_reqs_from": set(),   # 'inherits_params_from' is an empty set
			                   "inherits_params_from": set()}
			#print "newClass " + c
			#print self.classes


	def addParameterToClass(self, p, c):
		self.newClass(c)
		params = self.classes[c].get("params", [])
		if not (p in params):
			params.append(p)
			self.classes[c]["params"] = params


	def addParamsFromOneClassToAnother(self, cfrom, cto):
		for p in self.classes[cfrom].get("params", []):
			#print "Adding params from " + cfrom + " to " + cto
			self.addParameterToClass(p, cto)



	def addRequirementToClass(self, reqparam, reqval, c):
		#print "\naddRequirementToClass " + reqparam + " " + reqval + " " + c
		self.newClass(c)
		reqs = self.classes[c].get("reqs", ())
		newreq = ((reqparam, reqval),)
		if newreq[0] not in reqs:
			reqs = reqs + newreq
			self.classes[c]["reqs"] = reqs


	# the purpose of collecting the class details was to add some new
	# optional parameter details - we do that here, adding the information
	# from self.classes to self.optionalparams
	# {"symbol_text_font_name": (("symbol_type", "text"), ("symbol_type", "both"))}
	def addClassDetailsToOptionalParameters(self):
		for c in self.classes:
			reqs = self.classes[c].get("reqs", ())
			params = self.classes[c].get("params", ())
			for p in params:
				x = reqs
				if x != ():
					self.addOptionalParam(p, x)
					#self.addOptionalParam(p, (x,))





	# recusrive function to deal with class inheritance
	def addRecursiveInheritedClassRequirements(self, classname, reclevel):
		spaces = " " * reclevel
		#print spaces + "rec: " + classname
		i = self.classes[classname].get('inherits_reqs_from', None)
		#print 'i (all inherited classes) for class ' + classname
		#print i
		#print spaces + "enter rec, class = " + classname
		for parent in i:
			#print parent
			if parent != None:
				reqs = self.addRecursiveInheritedClassRequirements(parent, reclevel+1)
				#print spaces + "reqs: "
				#print reqs
				if reqs != None:
					for req in reqs:
						#print "ADDING REQ FROM RECURSIVE"
						self.addRequirementToClass(req[0], req[1], classname)
				#self.addParamsFromOneClassToAnother(parent, classname)

		#print spaces + "escape clause"
		reqs = self.classes[classname].get("reqs", None)
		return reqs



	# recusrive function to deal with class inheritance
	def addRecursiveInheritedClassParams(self, classname, reclevel):
		spaces = " " * reclevel
		#print spaces + "rec: " + classname
		i = self.classes[classname].get('inherits_params_from', ())
		#print 'i (all inherited classes) for class ' + classname
		#print i
		#print spaces + "enter rec, class = " + classname
		for parent in i:
			#print parent
			if parent != None:
				params = self.addRecursiveInheritedClassParams(parent, reclevel+1)
				#print spaces + "reqs: "
				#print reqs
				if params != None:
					for params in params:
						#print "ADDING REQ FROM RECURSIVE"
						self.addParamsFromOneClassToAnother(parent, classname)

		#print spaces + "escape clause"
		params = self.classes[classname].get("params", None)
		return params



	def addInheritedClassRequirements(self):
		#print '^^^^^^'
		for c in self.classes:
		#	print "addInheritedClassRequirements: " + c
			self.addRecursiveInheritedClassRequirements(c, 1)

	def addInheritedClassParams(self):
		#print '&&&&&&&&&'
		for c in self.classes:
		#	print "addInheritedClassParams: " + c
			self.addRecursiveInheritedClassParams(c, 1)


	def startElement(self, name, attrs):
		if (name == "magics") :
			return

		if (name == "class"):
			self.classname = attrs.get("name")
			if (self.debug) :
				print("class: " + self.classname)
			#if attrs.get("metview") :
			#	self.classname = attrs.get("metview")

			self.newClass(self.classname)

			doc_inherits = attrs.get("doc_inherits", None)
			self.inherits = attrs.get("inherits")
			if doc_inherits != 'no' and self.inherits != '' and self.inherits != None:
				self.classes[self.classname]["inherits_params_from"].add(self.inherits)
				self.classes[self.classname]["inherits_reqs_from"].add(self.inherits)
				fname = "%s/%s.xml" % (sys.argv[1], self.inherits)
				if not (fname in self.filehistory):  # don't open the same file twice
					try:
						file = open(fname, "r")
						if (self.debug):
							print("Opened (start class)" + fname)
						self.filehistory.append(fname)
						object = ObjectHandler()
						object.myoptions = []
						object.myrules = {}
						object.mydef = []
						object.filehistory = self.filehistory
						object.classes = self.classes
						object.setTopLevel(False)
						parser = make_parser()
						parser.setContentHandler(object)
						parser.parse(file)
						for s in object.mydef:
							self.mydef.append(s)
						for s in object.myrules:
							self.myrules[s]  = object.myrules[s]
						for s in object.myoptions:
							self.myoptions.append(s)
						self.classes = object.classes
					except:
						#print "Could not open file " + fname
						pass

		if (name == "parameter") or (name == "metview_parameter"):
			self.ignore = False
			self.param = attrs.get("name")
			if (self.debug) :
				print("  param: " + self.param)
			if (attrs.get("implemented") == 'no'):
				return
			if (attrs.get("metview") == 'no'):
				return
			if (attrs.get("visible") == 'no') or (attrs.get("visible") == 'off') or (attrs.get("visible") == 'false'):
				return
			if (attrs.get("inherit_parent_reqs") != 'no'):
				docclass = attrs.get("doc_class", None)
				if docclass != None:
					paramclass = docclass
				else:
					paramclass = self.classname
				self.addParameterToClass(self.param, paramclass)
			newtype = attrs.get("to")
			metview_type = attrs.get("metview_interface")
			if metview_type != None:
				newtype = metview_type
			if (newtype in self.types):
				f = self.types[newtype]
				self.newparam(self.param, f(self, attrs), self.default(attrs))
			else:
				self.last = self.last + self.options(attrs)

		if (name == "set"):
			setname = attrs.get("name")
			if setname != "" :
				valuestring = attrs.get("value")
				values = valuestring.split('/')
				for val in values:
					self.addOptionalParam(setname, ((self.param ,val),))
				#print('opt: ', self.optionalparams)
				#print  "%s %s = %s %s " % ('%if', self.param.upper(), attrs.get("value").upper(), '%then')
				#print "\t%s %s " % ('%unset', attrs.get("name").upper())
				pass

		if (name == "option"):
				if (self.debug) :
					print("    option: "+ attrs.get("name") + " - " + attrs.get("fortran"))
				if attrs.get("visible") != 'no'  :
					self.last = self.last + "\t\t%s; %s\n" % (attrs.get("fortran").upper(), attrs.get("fortran").upper())
					self.addRequirementToClass(self.param, attrs.get("fortran"), attrs.get("name"))
					#if attrs.get("doc_inherits") != 'no':
					#	self.classes[attrs.get("name")]["inherits_reqs_from"].add(self.classname)
					#	print "YClass " + attrs.get("name") + " inherits_reqs_from " + self.classname
					if attrs.get("docdive") != 'no' and  attrs.get("doc_inherits") != 'no' :
						if attrs.get("xmlfile") is not None:
							fname = "/%s.xml" % attrs.get("xmlfile")
						else:
							fname = "%s/%s.xml" % (sys.argv[1], attrs.get("name"))
						if not (fname in self.filehistory):  # don't open the same file twice
							try:
								file = open(fname, "r")
								self.filehistory.append(fname)
								if (self.debug):
									print("Opened (start option) " + fname)
								object = ObjectHandler()
								object.myoptions = []
								object.myrules = {}
								object.mydef = []
								object.unset = {}
								object.filehistory = self.filehistory
								object.classes = self.classes
								object.setTopLevel(False)
								parser = make_parser()
								parser.setContentHandler(object)
								parser.parse(file)
								s = object.mydef
								id = "%s_%s" % (attrs.get("name"), attrs.get("fortran"))
								id = "%s" % ( attrs.get("fortran"))
								self.myrules[id] = s
								self.myoptions.append(s)
								self.classes = object.classes
								#print "objlast: " + object.last
								#print "selflast: " + self.last
								#self.inheritedlast = self.inheritedlast + object.last
								#self.last = self.last + object.last
								#print "SL2: " + self.last
							except:
								#print "********EXCEPT 1!!!!!"
								#print "Could not open file " + fname
								#print "Therefore adding to myoptions: "
								#print(self.mydef)
								#self.myoptions.append(self.mydef)
								#self.mydef = []
								self.myoptions.append([])
								pass
					else:
						self.myoptions.append([])
						#self.myoptions.append(self.mydef)
						#self.mydef = []



	def endElement(self, name):

		#if (name == "magics") :
		#	return

		if self.ignore:
			return

		if ((name == "parameter" or name == "metview_parameter") and len(self.myoptions) != 0) :
		#if (name == "parameter") :
			#print(self.myoptions)
			self.last = self.last + "\t} = %s\n" % self.defparam
			self.newparam(self.param, self.last, self.defparam)
			if (self.debug) :
				print("  endparam: " + self.param)
				print("  endparam SL: \n" + self.last)
			self.last = ""
			for option in self.myoptions:
				for p in option:
					#print "    adding newparam from option: " + p[0]
					self.newparam(p[0], p[1], p[2])
			self.myoptions = []
			for rules in self.myrules:
				current = rules
				unsets = []
				for unset in self.myrules:
					if (unset != rules):
						for p in self.myrules[unset]:
							try:
								unsets.remove(p[0])
							except:
								pass
							unsets.append(p[0])

				for unset in self.myrules:
					if (unset == rules):
						for p in self.myrules[unset]:
							try:
								unsets.remove(p[0])
							except:
								continue

				if  len(unsets) != 0 :
						#print  "%s %s != %s %s " % ('%if', self.param.upper(), current.upper(), '%then')
						#for unset in unsets:
						#	print "\t%s %s " % ('%unset', unset.upper())
						pass

			self.myrules = {}


		if (name == "magics") :
			if self.toplevel:
				#print "SL (end class): " + self.last
				#print('opt: ', self.optionalparams)
				definition = open(sys.argv[3], "w")
				definition.write("PARAMSHARE ; ParamShare; PARAMSHARE\n");
				definition.write("{\n\tCOLOUR {\n\t\t%include MagicsColors.h\n\t}\n")
				definition.write("\tSTYLE {\n")
				definition.write("\t\tSOLID; SOLID\n")
				definition.write("\t\tDASH; DASH\n")
				definition.write("\t\tDOT; DOT\n")
				definition.write("\t\tCHAIN_DOT; CHAIN_DOT\n")
				definition.write("\t\tCHAIN_DASH; CHAIN_DASH\n")
				definition.write("\t}\n")
				definition.write("\tFONT {\n")
				definition.write("\t\tARIAL; ARIAL\n")
				definition.write("\t\tCOURIER; COURIER\n")
				definition.write("\t\tHELVETICA; HELVETICA\n")
				definition.write("\t\tTIMES; TIMES\n")
				definition.write("\t\tSERIF; SERIF\n")
				definition.write("\t\tSANSSERIF; SANSSERIF\n")
				definition.write("\t\tSYMBOL; SYMBOL\n")
				definition.write("\t}\n")
				definition.write("}\n\n")

				definition.write("%s; Magics; Automatically generated\n{\n" % sys.argv[4])
				for param in self.mydef:
					#print('PARAM START')
					#print(param[1])
					definition.write("%s" % param[1])
					definition.write("\n")
					#print('PARAM END')
				#definition.write(self.inheritedlast)
				definition.write("}\n")
				definition.close()

				#print "------------------------------------------"
				#for cl in self.classes.keys():
				#	print cl + " : "
				#	print self.classes[cl]
				#print self.classes

				#print "((((((((((((((((((((((((((((((((((((((("
				#print self.optionalparams['contour_shade_technique']

				#print "+++++++++++++++++++++++++++++++++++++++++++"
				#print self.optionalparams
				#print "********************************************"
				self.addInheritedClassParams()
				self.addInheritedClassRequirements()
				self.addClassDetailsToOptionalParameters()
				self.addRelatedOptionalRequirements()
				#print "+++++++++++++++++++++++++++++++++++++"
				#print self.optionalparams
				self.addRelatedOptionalRequirements2()
				#print "********************************************"
				#for o in self.optionalparams:
				#	PRint o
				#	print self.optionalparams[o]
				#print self.optionalparams

				# two different ways to generate the rules
				if False:
					rules = open(sys.argv[5], "w")
					for optparam in self.optionalparams.keys():
						rules.write("\n%if")
						first = True
						prevparam = ""
						#print "conditionzzzz"
						#print self.optionalparams[optparam]
						for condition in self.optionalparams[optparam]:
							if not first:
								if condition[0] == prevparam:
									rules.write(" %and")
								else:
									rules.write(" %or")
							first = False
							prevparam = condition[0]
							rules.write(" " + condition[0].upper() + " != " + condition[1].upper())
						rules.write(" %then")
						rules.write("\n\t%unset " + optparam.upper())
						rules.write("\n")
					rules.close()

				else:
					self.reorganiseOptionalParameters()
					#print "\n\n==============================\n"
					#print self.optionalparams2
					self.ensureOptionalParameters2DependenciesAreInRightOrder()
					#print "\n\n..............................\n"
					#print self.optionalparams3

					rules = open(sys.argv[5], "w")
					#for conditions in self.optionalparams2:
					for clause in self.optionalparams3:
						#print "CLAUSE"
						#print clause
						conditions = clause[0]
						#for conditions in clause[0]:
						#print "CONDS: "
						#print conditions
						lconditions = list(conditions)
						lconditions.sort()
						#print "LCONDS: "
						#print lconditions
						rules.write("\n%if")
						first = True
						prevparam = ""
						#print('conditionsssss :')
						#print conditions
						#print type(conditions)
						#print len(conditions)
						for condition in lconditions:
							#print "condition:"
							#print condition
							#print type(condition)
							#print len(condition)
							if not first:
								if condition[0] == prevparam:
									rules.write(" %and")
								else:
									rules.write(" %or")
							first = False
							prevparam = condition[0]
							#print('condition rules:')
							#print condition
							rules.write(" " + condition[0].upper() + " != " + condition[1].upper())
						rules.write(" %then")
#						for optparam in self.optionalparams2[conditions]:
						for optparam in clause[1]:
							rules.write("\n\t%unset " + optparam.upper())
						rules.write("\n")
					rules.close()


object = ObjectHandler()
saxparser = make_parser()
saxparser.setContentHandler(object)

datasource = open(sys.argv[1] +"/" + sys.argv[2], "r")
saxparser.parse(datasource)



# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

from Magics.macro import *

#Loading GRIB file
data = mgrib(grib_input_file_name='CAPE.grib',grib_field_position=1)

#Setting output
output = output(
	output_formats                = ['png'],
	output_name                   = 'contour15',
	output_name_first_page_number = 'off')

#Setting the geographical area
area = mmap(
	subpage_lower_left_latitude    = -90,
	subpage_lower_left_longitude   = -180,
	subpage_map_projection         = 'cylindrical',
	subpage_upper_right_latitude   = 90,
	subpage_upper_right_longitude  = 180,
)    

#Setting the coastlines
background = mcoast(
	map_coastline_land_shade        = 'on',
    map_coastline_land_shade_colour = 'cream')

foreground = mcoast(
	map_coastline_land_shade        = 'off',
	map_grid_line_style             = 'dash',
	map_grid_colour                 = 'grey',
	map_label                       = 'on',
	map_coastline_colour            = 'black')

#Defining the contour
contour = mcont(
	contour                        = 'off',
	contour_hilo                   = 'off',
	contour_interval               = 200.0,
	contour_label                  = 'off',
	contour_level_list             = [100.0, 200.0, 300.0, 500.0, 750.0, 1000.0, 1250.0, 1500.0, 1750.0, 2000.0, 2250.0, 2500.0, 2750.0, 3000.0, 3500.0, 4000.0, 4500.0],
	contour_level_selection_type   = 'level_list',
	contour_shade                  = 'on',
	contour_shade_colour_list      = ['rgb(1.00,0.35,0.00),rgb(1.00,  0.55,  0.00)', 'rgb(1.00,  0.67,  0.00)', 'rgb(1.00,  0.78,  0.00)', 'rgb(1.00,  0.89,  0.00)', 'rgb(1.00,  1.00,  0.00)', 'rgb(0.75,  0.98,  0.00)', 'rgb(0.60,  0.86,  0.10)', 'rgb(0.43,  0.77,  0.20)', 'rgb(0.30,  0.68,  0.40)', 'rgb(0.50,  0.90,  1.00)', 'rgb(0.50,  0.70,  1.00)', 'rgb(0.37,  0.50,  1.00)', 'rgb(0.20,  0.30,  0.90)', 'rgb(0.10,  0.15,  0.75)', 'rgb(0.00,  0.05,  0.55)', 'rgb(0.00,  0.00,  0.35)', 'rgb(0.00,  0.00,  0.20)'],
	contour_shade_colour_method    = 'calculate',
	contour_shade_label_blanking   = 'on',
	contour_shade_max_level        = 4200.0,
	contour_shade_method           = 'area_fill',
	contour_shade_min_level        = 100.0,
	legend                         = 'on',
)

#Picking the grib metadata
title = mtext(
    text_lines                     = ['<font size="1">CAPE - Shade from 100 to 4500 (orange-blue)</font>','<magics_title/>'],
    text_justification             = 'left',
    text_font_size                 = 0.6,
    text_colour                    = 'charcoal')     

#Plotting
plot(output,area,background,data,title,contour,foreground)

#For documentation only
tofortran('contour15',output,area,data,background,contour,foreground,title)
tomv4('contour15',contour)
tohtml('contour15',data,contour)


# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

# importing Magics module

from Magics.macro import *

ref = 'graph9'

# Setting of the output file name

output = output(output_formats=['png'],
                output_name_first_page_number='off',
                output_name=ref)

# Setting the cartesian view

projection = mmap(
    subpage_y_position=2.,
    subpage_map_projection='cartesian',
    subpage_x_axis_type='date',
    subpage_y_axis_type='regular',
    subpage_x_date_min='2012-03-01 12:00:00',
    subpage_x_date_max='2012-03-07 12:00:00',
    subpage_y_min=30.,
    subpage_y_max=75.,
    )

# Vertical axis

vertical = maxis(
    axis_orientation='vertical',
    axis_type='regular',
    axis_tick_label_height=0.40,
    axis_tick_label_colour='navy',
    axis_grid='on',
    axis_grid_colour='grey',
    axis_grid_thickness=1,
    axis_grid_line_style='dot',
    )

# Horizontal axis

horizontal = maxis(
    axis_orientation='horizontal',
    axis_type='date',
    axis_grid='on',
    axis_days_label_height=0.40,
    axis_months_label_height=0.40,
    axis_years_label_height=0.50,
    axis_grid_colour='grey',
    axis_grid_thickness=1,
    axis_grid_line_style='dot',
    )

# define  the  data

x = ['2012-03-02 00:00:00', '2012-03-02 12:00:00',
     '2012-03-03 00:00:00', '2012-03-04 00:00:00', 
     '2012-03-04 12:00:00',	'2012-03-05 00:00:00', 
     '2012-03-06 12:00:00', '2012-03-07 00:00:00' ]
y1 = numpy.array([35., 45., 40., 45., 40., 30.,35., 45.,])
y2 = y1+5
y3 = y1+10

solid_input = minput(input_date_x_values =x,
	input_y_values=y1)

# Define the graph

solid = mgraph(
    graph_type='bar',
    graph_bar_justification='centre',
    graph_bar_colour='cyan',
    graph_bar_width=3 * 3600.,
    legend='on',
    graph_shade_style = "solid",
    legend_user_text="<font colour='cyan'> solid </font>"
        ,
    )
# define  the  data
dot_input = minput(input_date_x_values =x,
	input_date_x2_values =x,
	input_y_values=y1,
	input_y2_values=y2)
	
# define  the  graph
dot = mgraph(
    graph_type='bar',
    graph_bar_justification='centre',
    graph_bar_colour='green',
    graph_bar_width=3 * 3600.,
    legend='on',
    graph_shade_style = "dot",
    graph_shade_dot_size = 0.02,
    graph_shade_dot_density = 20,
    legend_user_text="<font colour='green'> dot </font>"
        ,
    )

# define  the  data
hatch_input = minput(input_date_x_values =x,
	input_date_x2_values = x,
	input_y_values=y2,
	input_y2_values=y3)

hatch = mgraph(
    graph_type='bar',
    graph_bar_justification='centre',
    graph_bar_colour='pink',
    graph_bar_width=3 * 3600.,
    legend='on',
    graph_shade_style = "hatch",
    graph_shade_hatch_index = 4,
    legend_user_text="<font colour='pink'> hatch </font>"
        ,
    )
# Define the graph
title = mtext(text_lines=['Shading and Bar...'],
              text_justification='left', text_font_size=1.,
              text_colour='charcoal')

# To the plot
plot(
    output,
    projection,
    vertical,
    horizontal,
    solid_input, solid,
    dot_input, dot,
    hatch_input, hatch,
    title,
    )
#for the documentation
tofortran (
	ref, 
    output,
    projection,
    vertical,
    horizontal,
     solid_input, solid,
    dot_input, dot,
    hatch_input, hatch,
    title,
    )

tomv4 (
	ref, 
    solid,
    dot,
    hatch,
	)
tohtml (
	ref, 
    solid_input, solid,
    dot_input, dot,
	)



#!/usr/bin/env python 
# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

from xml.sax.handler import ContentHandler
from xml.sax import make_parser
from types import *

import sys
import json


class ObjectHandler(ContentHandler):
	path ="../src/xml/"
	tag = ""
	doc = ""
	magics = {}
	objects = {}
	object = {}
	param = {}
	paramdoc = ""
	actiondoc = ""
	parameters = {}

	def boolset(p, attrs):
		p["set"] = "psetc"
		p["values"] = ["on", "off"]

	def lineset(p, attrs):
		p["set"] = "psetc"
		p["values"] = ["solid", "dash", "dot", "chain_dash", "chain_dot"]
		p["type"] = "toggle"

	def justification(p, attrs):
		p["set"] = "psetc"
		p["values"] = ["left", "right", "centre"]
		p["type"] = "toggle"

	def listpolicy(p, attrs):
		p["set"] = "psetc"
		p["values"] = ["lastone", "cycle"]
		p["type"] = "toggle"
		
	def anyset(p, attrs):
		p["set"] = "psetc"
		if  attrs.has_key("values") :
			val = attrs["values"]
			values = val.split("/")
			p["values"] = values
			p["type"] = "toggle"
		return
	
	
	def intnumber(p, attrs):
		p["type"] = "number"
		p["set"] = "pseti"
		return
	def floatnumber(p, attrs):
		p["type"] = "number"
		p["set"] = "psetr"

	def intlist(p, attrs):
		p["type"] = "numberlist"
		p["set"] = "pset1i"
		return

	def floatlist(p, attrs):
		p["type"] = "numberlist"
		p["set"] = "pset1r"
		return

	def stringlist(p, attrs):
		p["type"] = "stringlist"
		p["set"] = "pset1c"
		return

	types = { 
		"bool": boolset,
		"float": floatnumber,
		"int": intnumber,
		"floatarray": intlist,
		"intarray": floatlist,
		"stringarray": stringlist,
		"string": anyset,
		"AxisAutomaticSetting": anyset,
		"Colour": anyset,
		"LineStyle": lineset,
		"ListPolicy": listpolicy,
		"Justification": justification
	}


	def get(self, attrs, name, default):
		if  attrs.has_key(name):
			return attrs.get(name)
		else :
			return default

	def newparam(self, attrs):
		p = {}
		self.parameters[attrs.get("name")] = p
		p["name"] = attrs.get("name")
		p["type"] = attrs.get("to")
		if attrs.has_key("metview") :
			p["metview"]= attrs["metview"]
		p["values"] = []
		t = p["type"]
		if  self.types.has_key(t) :
			self.types[t](p, attrs)
		else :
			p["set"]="psetc"
			p["type"]="toggle"


		p["default"] = attrs.get("default")
		return p

	def newobject(self, attrs):
		o = {}
		o["name"] = attrs.get("name")
		o["documentation"] = ""
		o["parameters"] = []
		if  attrs.has_key("inherits") :
			o["inherits"]=attrs.get("inherits")
			
		return o

	def startElement(self, name, attrs):
		self.tag = name
		if ( name == "class" ) : 
			self.doc = name
		# New object
			self.object = self.newobject(attrs)
			self.magics[attrs.get("name")] = self.object;
		if ( name == "parameter" ) : 
			if  self.get(attrs, "visible", "on") == "on" :
				self.doc = name
				self.param = self.newparam(attrs)
				self.object["parameters"].append(self.param)
			else :
				self.doc = "nodoc"
		if ( name == "option") : 
			val = attrs.get("fortran")
			self.param["values"].append(val)
			if ( self.param.has_key(val) == False ):
				self.param[val] = []
			self.param[val].append(attrs.get("name"))
		if ( name == "set" ) : 
			val = attrs.get("value")
			if ( self.param.has_key(val) == False ):
				self.param[val] = []
			self.param[val].append(attrs.get("name"))

		

	def characters(self, content):
		if ( self.tag == "documentation" ) : 
			if ( self.doc == "class" ) :
				self.actiondoc += content.lstrip();
			if ( self.doc == "parameter" ) :
				self.paramdoc += content.lstrip();
				

	def endElement(self, name):
		if ( name == "parameter"  and self.doc == "parameter") : 
			self.param["documentation"] = self.paramdoc
			self.paramdoc= ""
		if ( name == "class" ) : 
			self.object["documentation"] = self.actiondoc
			self.actiondoc= ""
			
	def printDef(self):
		print json.dumps(self.magics, indent=2)
	

	def parse(self, file):
		saxparser = make_parser()
		saxparser.setContentHandler(self)
		datasource = open(self.path + file, "r")
		saxparser.parse(datasource)
		return self.magics
	

	def prepare(self, defparam):
		param = defparam.copy()
		print "prepare-->"
		print param
		print "<---prepare"
		if param.has_key("values") == False:
			return param
		if len(param["values"]) == 0:
			return param
		for val in param["values"]:
			print val + " For " + param["name"]
			newp = []
			if param.has_key(val):
				for v in param[val]:
					print "prepare-->" + v
					# Can be a parameter or another object!
					if self.parameters.has_key(v) :
						self.append(newp, v)
					else:
						self.append(newp, self.getList(v))
			param[val] = newp

		return param

	def append(self, p1, p2):
		for p in p2:
			p1.append(p)


	def getList(self, object):
		definition = self.magics[object] 
		list = []
		if definition.has_key("inherits"):
			self.append(list, self.getList(definition["inherits"]))
		for p in definition["parameters"]:
			list.append(p["name"])
		return list;
			
			

	def buildParam(self, param):
		parameters = []
		if param.has_key("inherits"):
			print param["name"] + " inherits from " + param["inherits"]	
			parent = self.buildParam(self.magics[param["inherits"]])
			self.append(parameters, parent) 
		parameters.append(self.prepare(param))
		return parameters
				
		

	def build(self, params):
		parameters = []
		for p in params:
			print "find class " + p
			if ( self.magics.has_key(p) ) :
				defp = self.magics[p]
			# internal object
				for p in defp["parameters"]:
					self.append(parameters, self.buildParam(p))
		return parameters

	def getParams(self, action):
		return self.build(action);

	def getDoc(self, action):
		action = self.magics[action]

		return action["documentation"]
			


def createAction(abbr, name, files):

	object = ObjectHandler()
	for f in files:
		object.parse(f)

	parameters = object.getParams([name]);
	doc = object.getDoc(name);


	magics={}
	magics["magics"] = []
	action={}
	action["action"]= abbr;
	action["metview"]="m" + abbr
	action["fortran"]="p" + abbr
	action["documentation"]=doc
	action["parameters"]=parameters
	magics["magics"].append(action)



	f = open(abbr+'.json', "w")
	f.write(json.dumps(magics, indent=1))


#createAction("coast", "Coastlines",  ["Coastlines.xml", "CoastPlotting.xml", "LabelPlotting.xml", "GridPlotting.xml"]) 
#createAction("axis",  "Axis", ["Axis.xml"]) 
#createAction("symb",  "SymbolPlotting", ["SymbolPlotting.xml"]) 
#createAction("subpage",  "FortranViewNode", ["SubPage.xml", "GeoRectangularProjection.xml", 
#											 "XYTransformation.xml", "Proj4Projection.xml", "PolarStereographicProjection.xml"]) 

createAction("wind",  "Wind", ["wind.xml"]) 



# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

# importing Magics module

from Magics.macro import *

ref = 'axis3'

# Setting of the output file name

output = output(output_formats=['png'],
                output_name_first_page_number='off', output_name=ref)

# Setting the cartesian view

projection = mmap(
    subpage_y_position=2.,
    subpage_x_position=3.,
    subpage_map_projection='cartesian',
    subpage_x_axis_type='geoline',
    subpage_y_axis_type='logarithmic',
    subpage_x_min_latitude=-20.,
    subpage_x_max_latitude=20.,
    subpage_x_min_longitude=-180.,
    subpage_x_max_longitude=180.,
    subpage_y_min=1020.,
    subpage_y_max=10.,
    )

# Vertical axis

vertical = maxis(
    axis_orientation='vertical',
    axis_grid='on',
    axis_type='logarithmic',
    axis_tick_label_height=0.4,
    axis_tick_label_colour='navy',
    axis_grid_colour='grey',
    axis_grid_thickness=1,
    axis_grid_reference_level=0.,
    axis_grid_reference_line_style='solid',
    axis_grid_reference_thickness=1,
    axis_grid_line_style='dash',
    axis_title='on',
    axis_title_text='My Title',
    axis_title_font='times',
    axis_title_font_style='italic',
    axis_title_height=1.,
    )

# Horizontal axis

horizontal = maxis(
    axis_orientation='horizontal',
    axis_type='geoline',
    axis_tick_label_height=0.4,
    axis_tick_label_colour='navy',
    axis_grid='on',
    axis_grid_colour='grey',
    axis_grid_thickness=1,
    axis_grid_line_style='dash',
    )

lines = ['Using an horizontal geoline and vertical logarithmic axis...']

title = mtext({
    'text_lines': lines,
    'text_html': 'true',
    'text_justification': 'left',
    'text_font_size': 1.,
    'text_colour': 'charcoal',
    })

# To the plot
plot(output, projection, vertical, horizontal, title)

#For documentation only
tofortran(ref, output, projection, vertical,horizontal, title)
tomv4(ref, projection, vertical, horizontal)
tohtml(ref, projection, vertical, horizontal)




# (C) Copyright 1996-2019 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

import xarray as xr
import numpy as np

from Magics import macro as magics

ref = "xarray5"
ds = xr.open_dataset('C3S_OZONE-L4-TC-ASSIM_MSR-201608-fv0020.nc', decode_times=False)

png = magics.output(output_name_first_page_number = "off", output_name = ref)
data = magics.mxarray(xarray_dataset = ds, xarray_variable_name = "total_ozone_column")
contour = magics.mcont(contour_automatic_setting = "ecmwf")

magics.plot(png, data, contour, magics.mcoast())



import jinja2
from xml.sax.handler import ContentHandler
from xml.sax import make_parser
from datetime import date

import sys
import json


class ObjectHandler(ContentHandler):

	
	generated = date.today().strftime('%Y-%m-%d')
	magics = {}
	current = ""
	basic = {
		"bool" : "getBool",
		"int"  : "getInt",
		"float" : "getDouble",
		"string" : "getString",
		"stringarray" : "getStringArray",
		"intarray" : "getIntArray",
		"floatarray" : "getDoubleArray",
		"longintarray" : "getLongIntArray",
	}
	types = {
		"float" : "double",
		"floatarray" : "doublearray",
		"longintarray" : "longintarray",
	}
	
	
	def newclass(self, attrs):
		self.current = attrs.get("action", "unknown")
		self.parameters = self.magics.get(self.current, {})
	

	def parameter(self, attrs):
		if attrs.get("implemented") == "no" :
			return
		self.parameters.update( { attrs["name"] : { key : attrs[key] for key in ["from", "default"] }})

		
	options = { "class" : newclass,
				"parameter" : parameter,
			}		



	def startElement(self, name, attrs):
		if name in self.options.keys():
			self.options[name](self, attrs)
	def endElement(self, name):
		self.magics[self.current] = self.parameters
		

object = ObjectHandler()
saxparser = make_parser()
saxparser.setContentHandler(object)

import glob


for a in glob.glob("../src/params/*.xml") :
	datasource = open(a, "r")
	saxparser.parse(datasource)	

small = { key : object.magics[key] for key in ["unknown", "pcont"] }


print json.dumps(object.magics, indent = 4)





# (C) Copyright 2018- ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

#importing Magics module
from Magics.macro import *

ref = 'netcdf_contour_python'

#Setting of the output file name
output = output(output_formats = ['png'],
		output_name_first_page_number = "off",
		output_name = ref)

#Coastlines setting
coast = mcoast( map_grid =  "on",
                map_grid_colour  =  "tan",
				map_coastline_land_shade  =  'on',
				map_coastline_land_shade_colour  =  'cream',
				map_coastline_colour =  "tan")

#Import data
t2m = mnetcdf(
  netcdf_type = "geomatrix",
  netcdf_filename = "era5_2mt.nc",
  netcdf_value_variable = "t2m "
)

#Define the simple contouring
contour = mcont(legend = "off",
                contour_line_colour = "navy",
                contour_line_thickness =  2,
                contour_label =  "on",
                contour_highlight_colour =  "navy",
                contour_highlight_thickness =  6 )

#To the plot
plot(output, coast, t2m, contour)



# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.


#importing Magics module
from Magics.macro import *


ref = 'contour5'
#Setting of the output file name
output = output(output_formats = ['png'], 
		output_name_first_page_number = "off",
		output_name = ref)

#Setting the coordinates of the geographical area
projection = mmap(
				subpage_x_length = 24., 
				subpage_upper_right_longitude = 50.00,
        		subpage_upper_right_latitude = 65.00,
		        subpage_lower_left_latitude = 25.00,
			    subpage_lower_left_longitude = -20.00,
			    subpage_map_projection = 'cylindrical')



#Coastlines setting
coast = mcoast( map_grid =  "on",
                map_grid_colour  =  "grey",
                map_grid_thickness  =  2,
				map_coastline_colour =  "RGB(0.4,0.4,0.4)",
				map_coastline_thickness = 3)


#Import the t850 data 
t850 =  mgrib(grib_input_file_name  = "t850.grb",
              grib_field_position =  1)



#Define a contour using a predefined contour level_list
contour = mcont( legend = "on",
				contour_level_selection_type = "level_list",
				contour_level_list=[-20., -10., -5., -2.5, -1, -0.5, 0., 0.5, 1, 2.5, 5, 10, 20, 50],
                contour_line_colour = "grey",
                contour_line_thickness =  2,
                contour_label =  "off",
				contour_highlight="off",
                contour_shade = "on",
                contour_shade_method = "area_fill",
				contour_shade_colour_method = "calculate",
				contour_shade_max_level_colour = "red",
				contour_shade_min_level_colour = "blue",
				contour_shade_colour_direction = "clockwise"
				)
                


title = mtext(
           text_lines = ["<font size='1'>Using Shading and positional legend ...</font>",
		   	"<font colour='evergreen'>contour_level_selection_type = level_list</font> ", 
			"    uses a user-defined list of contour levels.",
			"[-20., -10., -5., -2.5, -1, -0.5, 0., 0.5, 1, 2.5, 5, 10, 20, 50]"],
		   text_justification = "left",
		   text_font_size = 0.8,
           text_colour =  "charcoal")


legend = mlegend(legend='on', 
				legend_display_type='continuous',
				legend_border='off', 
				legend_title='on', 
				legend_title_text='My Title',
				legend_title_orientation='vertical',
				legend_text_font_size=0.5,
				legend_text_colour="navy",
				legend_box_mode ='positional',
				legend_box_x_position = 26.,
				legend_box_x_length=2.,
				legend_box_y_position=2.,
				legend_box_y_length=10.
				)

#To the plot
plot(output, projection, t850, contour, coast, legend,title)

#For documentation only
tofortran(ref, output, projection, t850, contour, coast, legend,title)
tomv4(ref, contour)
tohtml(ref, t850, contour)

















# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.


#importing Magics module
from Magics.macro import *

#Example reference
ref = 'streamlines'

#Setting of the output file name
output = output(output_formats= ['png'],
                output_name_first_page_number= 'off',
                output_name= ref)

#Setting the coordinates of the geographical area
australia = mmap(subpage_upper_right_longitude= 180.,
                 subpage_upper_right_latitude= -5.,
                 subpage_lower_left_longitude= 100.,
                 subpage_map_projection= 'cylindrical',
                 subpage_lower_left_latitude= -55.)

#Background Coastlines 
background = mcoast( map_coastline_sea_shade_colour= 'white',
                     map_coastline_land_shade_colour= 'cream',
                     map_grid= 'off',
                     map_coastline_land_shade= 'on',
                     map_coastline_sea_shade= 'on',
                     map_label= 'off',
                     map_coastline_colour= 'tan')

#Foreground Coastlines
foreground = mcoast(  map_grid= 'on',
		      map_grid_colour = 'tan',
		      map_label= 'off',
		      map_coastline_colour= 'tan',
		      map_coastline_land_shade= 'off',
                      map_coastline_sea_shade= 'off')

#Import the  wind  at 200hPa uv200 
uv200 =  mgrib( grib_input_file_name = './uv200.grb',grib_id= 'uv200')

uv200_wind = mwind(
                legend= 'on',
                wind_field_type = 'streamlines',
		wind_arrow_unit_velocity = 40.0,
		wind_arrow_min_speed = 20.0,                
		wind_thinning_factor= 3.,
		wind_advanced_method = 'on',
		wind_advanced_colour_selection_type = 'interval',
		wind_advanced_colour_level_interval = 10.0,
		wind_advanced_colour_reference_level = 10.0,
		wind_advanced_colour_max_value = 100.0,
		wind_advanced_colour_min_value = 20.0,
		wind_advanced_colour_table_colour_method = 'calculate',
		wind_advanced_colour_direction = 'anti_clockwise',
		wind_advanced_colour_min_level_colour = 'turquoise',
		wind_advanced_colour_max_level_colour = 'purple_red')

title = mtext( text_lines = ["<font size='1'>Wind arrow colour is a function of speed</font>"],
	       text_justification = 'left',
	       text_font_size = 0.5,
	       text_colour = 'charcoal')

#add a legend
legend = mlegend(legend= 'on',
           legend_text_colour= 'charcoal',
           legend_box_mode= 'positional',
           legend_box_x_position= 27.5,
           legend_box_y_position= 4.,
           legend_box_x_length= 2.,
           legend_box_y_length= 12.,
           legend_border= 'off',
           legend_border_colour= 'charcoal',
           legend_box_blanking= 'on',
           legend_display_type= 'continuous',
           legend_title = 'on',
	   legend_title_text= 'Wind speed at 200 hPa',
	   legend_text_font_size = '0.5')

#To the plot
plot(output, australia, background,uv200, uv200_wind,foreground, title, legend)

#For documentation only
tofortran(ref, output, australia, background,uv200, uv200_wind,foreground, title, legend)
tomv4(ref,uv200_wind)
tohtml(ref,uv200_wind)



#!/usr/bin/env python
import jinja2
from xml.sax.handler import ContentHandler
from xml.sax import make_parser
from datetime import date

import sys



class ObjectHandler(ContentHandler):

    name = ""
    tag = ""
    generated = date.today().strftime('%Y-%m-%d')
    parameters = { "basic" : [],
                   "factory" : [] }
    basic = {
        "bool" : "getBool",
        "int"  : "getInt",
        "float" : "getDouble",
        "string" : "getString",
        "stringarray" : "getStringArray",
        "intarray" : "getIntArray",
        "floatarray" : "getDoubleArray",
        "longintarray" : "getLongIntArray",
    }
    types = {
        "float" : "double",
        "floatarray" : "doublearray",
        "longintarray" : "longintarray",
    }
    include = {}
    include_options = {}
    inherits = ""
    top = ""
    metview_default =""
    implements = []
    wrapper_include = []

    def addinterface(self, name):
        if name == "" :
            return
        add = name.split('/')
        for a in add:
            self.implements.append(a)

    def addimplements(self, name) :
        if name == "" :
            return
        add = name.split('/')
        for a in add:
            self.include["%s.h" % a] = "%s.h" % a
        self.addinterface(name)

    def newclass(self, attrs):
        self.name = attrs.get("name")
        self.prefix = attrs.get("prefix", "").split("/")
        self.tag = attrs.get("xmltag")
        self.metview_prefix = ""
        if "metview_prefix" in attrs.keys():
            self.metview_prefix = attrs.get("metview_prefix")

        if "inherits" in attrs.keys():
            self.inherits=attrs.get("inherits")
        self.abstract = "abstract" in attrs.keys()
        if "top" in attrs.keys():
            self.top=attrs.get("top")

        self.object_include = "%s.h" % self.name

        if "include" in attrs.keys():
            self.object_include = attrs.get("include")

        self.addimplements(attrs.get("implements", ""))
        self.addinterface(attrs.get("interface", ""))
        self.current = {}

    def parameter(self, attrs):
        if attrs.get("implemented") == "no" :
            return
        if attrs.get("metview") == "no":
            return

        type = attrs.get("to")

        fromt = attrs.get("from")
        to = self.types.get(type, type)

        default = attrs.get("default")
        if attrs.get("metview_default"):
            self.metview_default = attrs.get("metview_default")

        name = attrs.get("name")
        name = name.replace(self.metview_prefix, '')

        if type in self.basic :
            self.parameters["basic"].append (
                {
                    "name" : name,
                    "from" : self.types.get(fromt, fromt),
                    "to" : to,
                    "member" : attrs.get("member"),
                    "default" : default,
                    "method" : self.basic[type]
                }
                )
            if type in ["string", "bool"]:
                self.parameters["basic"][-1]["delimiter"] = "\""
        else :
            self.parameters["factory"].append (
                {
                    "name" : name,
                    "from" : attrs.get("from"),
                    "to" : attrs.get("to"),
                    "member" : attrs.get("member"),
                    "default" : attrs.get("default"),
                    "options"  : [],
                    "method" : "setMember",
                    "delimiter" : "\""
                }
                )
            if ( type in ["LineStyle", "ArrowPosition", "Justification",
                          "AxisAutomaticSetting", "ListPolicy", "Position",
                          "Hemisphere", "DisplayType", "Matrix", "cairo_t*"]):
                self.parameters["factory"][-1]["enum"] = True
                self.parameters["factory"][-1]["method"] = "setAttribute"
                self.parameters["factory"][-1]["niceprint"] = True

            else :
                if type not in ["Colour"]:
                    self.include[attrs.get("include", "%s" % attrs.get("to"))] = attrs.get("include", "%s" % attrs.get("to"))
                    self.wrapper_include.append(attrs.get("to"))

            if type in ["Matrix"]:
                self.parameters["factory"][-1]["delimiter"] = ""
                self.include["Matrix.h"] = "Matrix.h"
            if type in ["cairo_t*"]:
                self.parameters["factory"][-1]["delimiter"] = ""
                self.include["cairo.h"] = "cairo.h"
            if type in ["Colour"]:
                self.parameters["factory"][-1]["niceprint"] = True

    def option(self, attrs):

        param = self.parameters["factory"][-1]
        name = attrs.get("xml")
        if name != None:
            param["options"].append({
                "key" : attrs.get("xml"),
                "name" : name.replace(":", "_"),
                "parent" : param["to"],
                "object" : attrs.get("name") } )

            self.include_options[attrs["include"]] = attrs["include"]

        if attrs.get("fortran") != attrs.get("xml"):
            name = attrs.get("fortran")
            if name != None:
                param["options"].append({
                    "key" : attrs.get("fortran"),
                    "name" : name.replace(":", "_"),
                    "parent" : param["to"],
                    "object" : attrs.get("name") } )
                self.include_options[attrs["include"]] = attrs["include"]

    options = { "class" : newclass,
                "parameter" : parameter,
                "option" : option }

    def startElement(self, name, attrs):
        if name in self.options.keys():
            self.options[name](self, attrs)
    def endElement(self, name):
        pass

object = ObjectHandler()
saxparser = make_parser()
saxparser.setContentHandler(object)

toolssource = sys.argv[1]
datasource = open(sys.argv[2], "r")
destination = sys.argv[3]

saxparser.parse(datasource)

with open("%s/source_mv.template" % toolssource,  "r") as source:
    template = jinja2.Template(source.read())
with open("%s/%sWrapper.cc" % (destination, object.name), "wt") as out:
    out.write(template.render(object = object.name,
                              string_parameters = object.parameters["basic"],
                              factory_parameters = object.parameters["factory"],
                              include = object.include,
                              include_options = object.include_options,
                              date = object.generated,
                              tag = object.tag,
                              top = object.top,
                              metview_default = object.metview_default,
                              abstract = object.abstract,
                              inherit = object.inherits,
                              prefix = object.prefix
                             )
             )

with open("%s/header_mv.template" % (toolssource), "r") as source:
    template = jinja2.Template(source.read())
with open("%s/%sWrapper.h" % (destination, object.name), "wt") as out:
    out.write(template.render(object = object.name,
                              string_parameters = object.parameters["basic"],
                              factory_parameters = object.parameters["factory"],
                              include = object.include,
                              include_options = object.include_options,
                              implements = object.implements,
                              date = object.generated,
                              inherit = object.inherits,
                              tag = object.tag,
                              object_include = object.object_include,
                              wrapper = object.wrapper_include,
                              prefix = object.prefix
                             )
    )



# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.


#importing Magics module
from Magics.macro import *
import json

#Example reference
ref = 'geojson_input'

#Setting of the output file name
output = output(output_formats= ['png'],
                output_name_first_page_number= 'off',
                output_name= ref)

#Setting the coordinates of the geographical area
area = mmap(subpage_upper_right_longitude= 180.,
                 subpage_upper_right_latitude= 90.,
                 subpage_lower_left_longitude= -180.,
                 subpage_lower_left_latitude= -90.)

#Background Coastlines
background = mcoast( map_coastline_sea_shade_colour= 'white',
                     map_coastline_land_shade_colour= 'cream',
                     map_grid= 'off',
                     map_coastline_land_shade= 'on',
                     map_coastline_sea_shade= 'on',
                     map_label= 'off',
                     map_coastline_colour= 'tan')

#Foreground Coastlines
foreground = mcoast( map_grid= 'on',
                 map_grid_colour = 'tan',
                 map_label= 'off',
                 map_coastline_colour= 'tan',
                 map_coastline_land_shade= 'off',
                 map_coastline_sea_shade= 'off')

#Import the z500 data
data = {
  "coordinates": [
    [
      [
        -112.4751265395822,
        49.22009389532161
      ],
      [
        -116.62509597124857,
        53.309626538152095
      ],
      [
        -118.9151697373996,
        49.43534662203797
      ]
    ],
    [
      [
        -125.27468276491868,
        -70.54530630794772
      ],
      [
        -130.27468276491868,
        -80.54530630794772
      ],
      [
        -128.69964317766592,
        -79.42844281752712
      ]
    ],
    [
      [
        106.53537282393648,
        45.173527191962684
      ],
      [
        110.31188673144166,
        48.952815714617586
      ],
      [
        105.10685624590496,
        56.823948318072986
      ],
      [
        100.89710323992226,
        60.471269981064836
      ],
      [
        120.61237115246117,
        48.225215772936494
      ]
    ]
   ],
  "properties": {
    "type": "warm fronts"
  },
  "type": "MultiLineString"
}

cold =  mgeojson( geojson_input_type= 'string',
        geojson_input = json.dumps(data),)
blue = mline(polyline_line_colour = "blue", polyline_line_thickness = 2)

title = mtext(text_lines = ["<font size='1'>GeoJSon First example</font>",
                        "",
                        "Warm fronts"],
          text_justification = 'left',
          text_font_size = 0.8,
          text_colour = 'charcoal')

#add a legend
legend = mlegend(legend= 'on',
           legend_text_colour= 'black',
           legend_box_mode= 'positional',
           legend_box_x_position= 15.,
           legend_box_y_position= 0.5,
           legend_box_x_length= 4.,
           legend_box_y_length= 18.,
           legend_border= 'off',
           legend_border_colour= 'black',
           legend_box_blanking= 'on',
           legend_display_type= 'continuous',
           legend_title = "on",
       legend_title_text= "Wind",
       legend_text_font_size = "0.5")

#To the plot
plot(output, background, cold, blue, foreground, title, legend)




# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.

import traceback
import sys

from gribapi import *
import json
from Magics.macro import *
import numpy
import datetime



reference = datetime.datetime.now()

def filename(path, param, type):
    return ( "%s/%s_%s.grib" % (path, type, param) )


def getdata(key, path, param, index, out):

    f = open( filename(path, param, key))

    mcount = grib_count_in_file(f)

    if key == "deterministic":
        key = "forecast"

    for i in range(mcount):
        gid = grib_new_from_file(f)
        if i == 0 :
            #param =  str(grib_get(gid, "parameter"))

            for l in range(len(index)):
                out[l][param][key] = []

        values = grib_get_elements(gid,"values",index)
        for l in range(len(index)):

            out[l][param][key].append(values[l])
        grib_release(gid)
    f.close()
    return


def control(path, param,  index, out ):


    f = open( filename(path, param, "control"))

    mcount = grib_count_in_file(f)

    for i in range(mcount):
        gid = grib_new_from_file(f)
        step = str(grib_get(gid, "step"))
        if i == 0 :
            #param =  str(grib_get(gid, "parameter"))
            date =  str(grib_get(gid, "date"))
            time =  "%02d" % grib_get(gid, "time")


            for l in range(len(index)):
                out[l][param] = {}
                out[l]["date"]  = date
                out[l]["time"]  = time
                out[l][param]["control"] = []
                out[l][param]["steps"] = []

        reference = datetime.datetime.strptime(date+time, '%Y%m%d%H%M')
        values = grib_get_elements(gid,"values",index)
        for l in range(len(index)):
            out[l][param]["control"].append(values[l])
            out[l][param]["steps"].append(step)
        grib_release(gid)
    f.close()
    return



def data(stations, param, path):
    out = []
    index = []

    lsm = open("%s/lsm.cf" % (path,) )

    detz_file = open("%s/det.z" % (path,) )
    epsz_file = open("%s/eps.z" % (path,) )
    gid = grib_new_from_file(lsm)
    detz = grib_new_from_file(detz_file)
    epsz = grib_new_from_file(epsz_file)


    for station in stations:
        nearest = grib_find_nearest(gid,station["lat"],station["lon"],1)
        name =  "%.2f_%.2f_%s.json" % (station["lat"], station["lon"], param)

        out.append({
            "station_name" : station["name"],
            "output" : name,
            "height": 8000.,
            "metadata": {
                "points_along_meridian" : grib_get(gid, "Nj"),
                "deterministic_height": 142.679443359,
                "eps_height": 149.973892212,
            },
            "location" : {  "lat" : station["lat"], "lon" : station["lon"] },
            "EPS_location" : {  "distance": nearest[0].distance, "latitude" : nearest[0].lat, "longitude" : nearest[0].lon },
            "forecast_location" : {  "distance": nearest[0].distance, "latitude" : nearest[0].lat, "longitude" : nearest[0].lon },
        })


        index.append(nearest[0].index)
        station[param] = name

    eps = grib_get_elements(epsz,"values",index)
    det = grib_get_elements(detz,"values",index)
    for l in range(len(index)):
            out[l]["metadata"]["deterministic_height"] = det[l]
            out[l]["metadata"]["eps_height"] = eps[l]

    grib_release(gid)
    grib_release(detz)
    grib_release(epsz)
    lsm.close()
    epsz_file.close()
    detz_file.close()



    control( path, param,  index, out )
    getdata("deterministic", path, param, index, out)
    getdata("max", path, param, index, out)
    getdata("min", path, param, index, out)
    getdata("median", path, param, index, out)
    getdata("ninty", path, param, index, out)
    getdata("seventyfive", path, param, index, out)
    getdata("twentyfive", path, param, index, out)
    getdata("ten", path, param, index, out)


    for i in out:
        f = open("%s.data" % (param, ), 'w')
        print >> f, json.dumps(i, indent=4, separators=(',', ': ') )



def epsgrams(location):

    z500 = "%.2f_%.2f_%s.json" % (location["lat"], location["lon"], "z500")
    precip = "%.2f_%.2f_%s.json" % (location["lat"], location["lon"], "precip")
    t850 = "%.2f_%.2f_%s.json" % (location["lat"], location["lon"], "t850")

    out = output(
        output_formats=location["output_formats"],
        output_name_first_page_number='off',
        output_name=str(location["output_name"]) ,
        super_page_y_length=29.7,
        super_page_x_length=21.,
        )

    y = 6.
    last = 21.5
    suby = 4.5
    # define the cartesian projection
    frame1 = page(
        layout='positional',
        page_x_length= 21.,
        page_y_length= y+3.,
        page_id_line='off',
        page_x_position=0.,
        page_y_position= last
        )
    last = last - y;
    projection = mmap(
        subpage_map_projection='cartesian',
        subpage_x_axis_type='date',
        subpage_x_automatic='on',
        subpage_y_axis_type='regular',
        subpage_y_automatic='on',
        subpage_y_length = suby,
        subpage_y_position = 0.5
        )
    font_size = 0.35
    # define horizontal axis

    horizontal = maxis(
        axis_orientation = "horizontal",
        axis_date_type = "days",
        axis_days_label = "both",
        axis_days_label_colour=  "navy",
        axis_days_label_height = font_size,
        axis_grid =  "on",
        axis_grid_colour = "navy",
        axis_grid_line_style= "dash",
        axis_line_colour= "navy",
        axis_minor_tick= "on",
        axis_minor_tick_colour= "navy",
        axis_months_label= "off",
        axis_tick_colour= "navy",
        axis_type =  "date",
        axis_years_label = "off"
        )

    # horizontal axis for bottom plot (with month and year)

    horizontal_bot = maxis(
        axis_orientation = "horizontal",
        axis_date_type = "days",
        axis_days_label = "both",
        axis_days_label_colour=  "navy",
        axis_days_label_height = font_size,
        axis_grid =  "on",
        axis_grid_colour = "navy",
        axis_grid_line_style= "dash",
        axis_line_colour= "navy",
        axis_minor_tick= "on",
        axis_minor_tick_colour= "navy",
        axis_months_label= "on",
        axis_months_label_colour=  "navy",
        axis_months_label_height = font_size,
        axis_tick_colour= "navy",
        axis_type =  "date",
        axis_years_label = "on",
        axis_years_label_colour=  "navy",
        axis_years_label_height = font_size
        )

    # define vertical axis

    vertical = maxis(
        axis_orientation = "vertical",
        axis_grid = "on",
        axis_grid_colour ="navy",
        axis_grid_line_style = "dash",
        axis_grid_reference_level =  0.,
        axis_grid_reference_thickness = 1,
        axis_line =  "on",
        axis_line_colour = "navy",
        axis_tick_colour = "navy",
        axis_tick_label_colour = "navy",
        axis_tick_label_height =  font_size
        )

    cc_data =   mwrepjson(
                        wrepjson_family =  "eps",
                        wrepjson_input_filename = "tcc.data",
                        wrepjson_parameter = "tcc",
                        wrepjson_parameter_information =  "Total Cloud Cover (okta) ",
                        wrepjson_parameter_scaling_factor =  8.,
                        wrepjson_product_information =  "HRES Forecast and ENS Distribution "
                    )
    cc_graph = mepsgraph( eps_box_border_thickness = 2,
                          eps_box_width = 1.5)
    cc_text = mtext(
                text_colour =  "navy",
                text_font_size = 0.2,
                text_justification =  "left",
                text_line_1 =  "<font size='0.5'>EPS Meteogram</font><font size='0.6'><json_info key='expver'/></font>",
                text_line_2 =  "<font size='0.5'><json_info key='station_name'/></font><font size='0.6'><json_info key='location'/></font> <font size='0.6'><json_info key='grid_point'/></font> <font size='0.6'><json_info key='height'/></font>",
                text_line_3 =  "<font size='0.5'><json_info key='product_info'/></font> <font size='0.6'><json_info key='date'/></font>",
                text_line_4 = "<font size='0.5' colour='white'>.</font>",
                text_line_5 =  "<font size='0.35'><json_info key='parameter_info'/></font>",
                text_line_count =  5,
            )

    frame2 = page(
            layout='positional',
            page_x_length=21.,
            page_y_length=y,
            page_id_line='off',
            page_x_position=0.,
            page_y_position=last
            )

    last  = last -y;

    pr_data = mwrepjson(
                        wrepjson_family =  "eps",
                        wrepjson_input_filename = "pr.data",
                        wrepjson_parameter = "pr",
                        wrepjson_parameter_information =  "Total Precipitation (mm/6h)",
                        wrepjson_parameter_scaling_factor =  1000.,
                        wrepjson_y_axis_threshold = 3.
                    )
    pr_graph = mepsgraph( eps_box_border_thickness = 2,
                          eps_box_width = 1.5,
                          eps_maximum_font_size =  0.35)

    ws_data = mwrepjson(wrepjson_family =  "eps",
                        wrepjson_input_filename = "ws.data",
                        wrepjson_parameter = "ws",
                        wrepjson_parameter_information =  "10m Wind Speed (m/s)",
                        )
    ws_graph = mepsgraph( eps_box_border_thickness = 2,
                          eps_box_width = 1.5,
                          )
    t2m_data = mwrepjson(

                        wrepjson_family =  "eps",
                        wrepjson_input_filename = "2t.data",
                        wrepjson_parameter = "2t",
                        wrepjson_parameter_information =  "2m Temperature(&deg;C)",
                        wrepjson_parameter_offset_factor = -273.15,
                        wrepjson_temperature_correction =  "on"
                    )
    t2m_graph = mepsgraph( eps_box_border_thickness = 2,
                          eps_box_width = 1.5,
                           eps_font_colour = "navy",
                        eps_legend_font_size =  font_size,
                        eps_grey_legend = "off",
                        legend='on'
                          )


    frame3 = page(
            layout='positional',
            page_x_length=21.,
            page_y_length=y,
            page_id_line='off',
            page_x_position=0.,
            page_y_position=last
            )

    last  = last -y;
    frame4 = page(
            layout='positional',
            page_x_length=21.,
            page_y_length=y,
            page_id_line='off',
            page_x_position=0.,
            page_y_position=last
            )

    text = mtext(
        text_lines=["<json_info key='parameter_info'/>"],
        text_html='true',
        text_colour='navy',
        text_font_size=font_size,
        #text_mode='positional',
        text_box_x_position=1.,
        text_box_y_position=5.1,
        text_box_x_length=10.,
        text_box_y_length=2.5,
        text_border='off',
        text_justification='left',
        )

    t2m_text = mtext(
        text_lines=["<json_info key='parameter_info'/><json_info key='full_temperature_correction_info'/>"],
        text_html='true',
        text_colour='navy',
        text_font_size=font_size,
        #text_mode='positional',
        text_box_x_position=1.,
        text_box_y_position=5.1,
        text_box_x_length=10.,
        text_box_y_length=2.5,
        text_border='off',
        text_justification='left',
        )
    legend = mlegend(
                legend_text_colour = "navy",
                legend_entry_text_width =  99.,
                legend_entry_plot_direction =  "row",
                legend_text_font_size =  font_size,
                legend_box_mode='positional',
                legend_box_x_position = 1.,
                legend_box_y_position = -4.5,
                legend_box_x_length   = 12.,
                legend_box_y_length   = 5.,
                )
    plot(
        out,
        frame1,
        projection,
        horizontal,
        vertical,
          cc_data, cc_graph, cc_text,
        frame2,
        projection,
        horizontal,
        vertical,
        pr_data, pr_graph, text,
        frame3,
        projection,
        horizontal,
        vertical,
        ws_data, ws_graph, text,
        frame4,
        projection,
        horizontal,
        vertical,
        t2m_data, t2m_graph, t2m_text,
        legend
        )

'''

'''
def main():

    if len(sys.argv) != 2 :
        print "No station list given"
        return 1

    try:
        stations = json.loads(open(sys.argv[1]).read())
        params = ["tcc", "pr", "ws", "2t"]
        for param in params :

            data(stations["stations"], param, "/tmp/cgs/epsdata/eps20160118000001/grib")
        for station in stations["stations"]:
            epsgrams(station)
        return 0


    except GribInternalError,err:
        if VERBOSE:
            traceback.print_exc(file=sys.stderr)
        else:
            print >>sys.stderr,err.msg

        return 1

if __name__ == "__main__":
    sys.exit(main())



# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.


#importing Magics module
from Magics.macro import *


ref = 'handling_missing1'
#Setting of the output file name
output = output(output_formats = ['png'],
        output_name_first_page_number = "off",
        output_name = ref)

area = [24.,-26.,71.,85.]
area = [24.,-26.,51.,78.]
area = [24.,-26.,71.,85.]

method = "interpolate"
fill = 5

#Setting the coordinates of the geographical area
projection = mmap(#subpage_map_projection = 'polar_stereographic',
                subpage_lower_left_latitude = area[0],
                subpage_lower_left_longitude = area[1],
                subpage_upper_right_latitude = area[2],
                subpage_upper_right_longitude = area[3],
                )



#Coastlines setting
coast = mcoast( map_grid =  "on",
                map_grid_colour  =  "tan",
                map_coastline_land_shade  =  'off',
                map_coastline_land_shade_colour  =  'cream',
                map_coastline_colour =  "tan")


#Import the z500 data
data =  mgrib(grib_input_file_name  = "sst.grb",
              grib_interpolation_method = method,
              grib_interpolation_method_missing_fill_count = fill,
              )

level = [-2.,0,2,4,6,8,10,12,14,16,18,20,22,24,26,28,30,32,34,36]
levels = []
for l in level:
    levels.append(l +273.15)


#Define the simple contouring for z500
contour = mcont(  legend               = "on",
          contour                      = "off",
          contour_shade                = "on",
          contour_level_selection_type = "level_list",
          contour_level_list           = levels,
          contour_shade_colour_method  = "list",

          contour_method= 'linear',
          contour_internal_reduction_factor = 2.,
          contour_shade_colour_list    = ["RGB(0.29,0.00,0.48)","RGB(0.61,0.00,1.00)","RGB(0.80,0.47,1.00)","RGB(0.00,0.00,1.00)",
                                          "RGB(0.00,0.35,1.00)","RGB(0.00,0.55,1.00)","RGB(0.04,0.49,0.00)","RGB(0.04,0.75,0.00)",
                                          "RGB(0.04,1.00,0.00)","RGB(0.63,0.61,0.00)","RGB(0.84,0.81,0.00)","RGB(1.00,0.93,0.00)",
                                          "RGB(0.66,0.33,0.00)","RGB(0.84,0.42,0.00)","RGB(1.00,0.52,0.00)","RGB(0.75,0.04,0.00)",
                                          "RGB(1.00,0.05,0.00)","RGB(1.00,0.52,0.50)","RGB(1.00,0.84,0.83)"],
          contour_shade_method         = "area_fill",
          contour_label                = "off"
  )





title = mtext(
           text_lines = ["Handling missing data : grib_interpolation_method=%s" % method],
           text_justification = "left",
           text_font_size = 0.6,
           text_colour =  "charcoal")


#To the plot
plot(output, projection, data, contour,  coast, title)

















# (C) Copyright 1996-2016 ECMWF.
# 
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
# In applying this licence, ECMWF does not waive the privileges and immunities 
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.



from xml.sax.handler import ContentHandler
from xml.sax import make_parser
import sys

class ObjectHandler(ContentHandler):
	title = ""
	name = ""
	tab1 = 0
	tab2 = 2
	open_file = 0

	def tab(self, n):
		sep=''
		i = 0
		while i < n:
			sep = sep + '\t'
			i = i +1
		return sep

	def nice(self, name, attrs):
		sep1 = self.tab(self.tab1)
		sep2 = self.tab(self.tab2)
		self.file.write("%s<%s " % (sep1, name))
		for key in attrs.getNames():
			self.file.write(" %s='%s'" % (key , attrs.getValue(key)))
		self.file.write(">")

	def characters(self, data):
		if self.open_file == 1:
			self.file.write(data);

	def startElement(self, name, attrs):
		print "found %s " % name
		if (name == "magics") :
			return
		if (name == "static") :
			return

		if (name == "class"):
			self.name = attrs.get("name")
			filename= "../params/%s.xml" % self.name
			self.file=open(filename, "w")
			self.open_file = 1;
			self.file.write("<magics>\n")
			print "open %s " % filename
		self.nice(name, attrs)
		self.tab1 = self.tab1+1
		self.tab2 = self.tab2+1

	def endElement(self,name):
		print "close %s\n " % name
		if ( self.open_file == 0):
			return

		self.tab1 = self.tab1-1
		self.tab2 = self.tab2-1
		self.file.write("%s</%s>" % (self.tab(self.tab1), name)) 
		if (name == "class") :
			self.file.write("\n</magics>\n")
			self.file.close()
			self.open_file = 0

object = ObjectHandler()
saxparser = make_parser()
saxparser.setContentHandler(object)

datasource = open(sys.argv[1],"r")
saxparser.parse(datasource)
